from .create import Create
from .get import Get
from .increase_guarantee import IncreaseGuarantee
from .list import List
from .payback import Payback

class Orders(Create, Get, IncreaseGuarantee, List, Payback):
  ...



from bit2me.core.mixin import Router
from .account import Account
from .currency import Currency
from .signin import Signin

class V3(Router):
  account: Account
  currency: Currency
  signin: Signin



from bit2me.core import Endpoint

class GetRewardsSummary(Endpoint):
  async def get_rewards_summary(
    self,
    wallet_id: str,
    *,
    user_currency: str | None = None,
    validate: bool = True
  ):
    """Retrieves full list of rewards summary for selected wallet
    
    - `wallet_id`: Wallet identifier
    - `user_currency`: Currency to show balance
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/wallets/{walletId}/rewards/summary)"""
    params = {}
    if user_currency is not None:
      params['userCurrency'] = user_currency
    r = await self.authed_request(
      'GET', f'/v1/earn/wallets/{wallet_id}/rewards/summary',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return r.json()



import os
from typing_extensions import Any, Mapping, Literal
from dataclasses import dataclass, field
import json as _json
import time
from urllib.parse import urlencode
import base64
import hashlib
import hmac
import httpx
from .client import HttpClient, HttpMixin, BIT2ME_API_URL

def message_to_sign(nonce: str, url: str, body: str | None = None) -> str:
  """Build the string that is SHA-256-hashed then HMAC-SHA512-signed.

  Bit2Me expects ``nonce:path`` when there is no body, and ``nonce:path:body`` when
  there is a JSON body (compact, no spaces). Do not append ``:`` for an empty body.
  """
  payload = f'{nonce}:{url}'
  if body is not None and body != '':
    payload += ':' + body
  return payload

def sign_message(message: str, secret: str) -> str:
  message_bytes = message.encode('utf-8')
  secret_bytes = secret.encode('utf-8')
  sha256 = hashlib.sha256()
  sha256.update(message_bytes)
  hash_digest = sha256.digest()
  hmac_sha512 = hmac.new(secret_bytes, hash_digest, hashlib.sha512)
  hmac_digest = base64.b64encode(hmac_sha512.digest()).decode()
  return hmac_digest

def auth_headers(api_key: str, secret_key: str, *, nonce: str | None = None, path: str, body: str | None = None) -> dict[str, str]:
  nonce = nonce or str(int(time.time() * 1000))
  msg = message_to_sign(nonce, path, body)
  signature = sign_message(msg, secret_key)
  return {
    'x-api-key': api_key,
    'api-signature': signature,
    'x-nonce': nonce
  }


@dataclass
class AuthHttpClient(HttpClient):
  api_key: str
  api_secret: str

  async def authed_request(
    self, method: str, base_url: str, path: str,
    *,
    json: Any | None = None,
    params: Mapping | None = None,
    headers: Mapping[str, str] | None = None,
    cookies: httpx._types.CookieTypes | None = None,
    auth: httpx._types.AuthTypes | httpx._client.UseClientDefault | None = httpx.USE_CLIENT_DEFAULT,
    follow_redirects: bool | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    timeout: httpx._types.TimeoutTypes | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    extensions: httpx._types.RequestExtensions | None = None,
  ):
    # Use `is not None` so JSON false / 0 / [] / "" still serialize and sign correctly.
    body = _json.dumps(json, separators=(',', ':')) if json is not None else None
    if params:
      path += '?' + urlencode(params, doseq=True)
    headers = {
      **auth_headers(self.api_key, self.api_secret, path=path, body=body),
      **(headers or {})
    }
    url = base_url + path
    return await self.request(
      method, url, headers=headers, content=body,
      auth=auth, follow_redirects=follow_redirects,
      cookies=cookies, timeout=timeout, extensions=extensions,
    )
  
  
@dataclass
class AuthHttpMixin(HttpMixin):
  base_url: str = field(kw_only=True, default=BIT2ME_API_URL)
  http: AuthHttpClient # type: ignore

  @classmethod
  def new(cls, api_key: str | None = None, api_secret: str | None = None, *, base_url: str = BIT2ME_API_URL):
    if api_key is None:
      api_key = os.environ['BIT2ME_API_KEY']
    if api_secret is None:
      api_secret = os.environ['BIT2ME_SECRET_KEY']
    client = AuthHttpClient(api_key=api_key, api_secret=api_secret)
    return cls(base_url=base_url, http=client)
  
  async def __aenter__(self):
    await self.http.__aenter__()
    return self
  
  async def __aexit__(self, exc_type, exc_value, traceback):
    await self.http.__aexit__(exc_type, exc_value, traceback)

  async def authed_request(
    self, method: str, path: str,
    *,
    json: Any | None = None,
    params: Mapping | None = None,
    headers: Mapping[str, str] | None = None,
    cookies: httpx._types.CookieTypes | None = None,
    auth: httpx._types.AuthTypes | httpx._client.UseClientDefault | None = httpx.USE_CLIENT_DEFAULT,
    follow_redirects: bool | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    timeout: httpx._types.TimeoutTypes | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    extensions: httpx._types.RequestExtensions | None = None,
  ):
    return await self.http.authed_request(
      method, self.base_url, path, params=params, headers=headers, json=json,
      auth=auth, follow_redirects=follow_redirects,
      cookies=cookies, timeout=timeout, extensions=extensions, 
    )

  async def authed_totp_request(
    self, method: str, path: str,
    *,
    totp_code: str, totp_type: Literal['gauth', 'sms', 'email'],
    json: Any | None = None,
    params: Mapping | None = None,
    headers: Mapping[str, str] | None = None,
    cookies: httpx._types.CookieTypes | None = None,
    auth: httpx._types.AuthTypes | httpx._client.UseClientDefault | None = httpx.USE_CLIENT_DEFAULT,
    follow_redirects: bool | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    timeout: httpx._types.TimeoutTypes | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    extensions: httpx._types.RequestExtensions | None = None,
  ):
    headers = {
      **(headers or {}),
      'x-totp': totp_code,
      'x-totp-type': totp_type,
    }
    return await self.http.authed_request(
      method, self.base_url, path, params=params, headers=headers, json=json,
      auth=auth, follow_redirects=follow_redirects,
      cookies=cookies, timeout=timeout, extensions=extensions, 
    )



from datetime import datetime
from typing_extensions import Any, Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  id: NotRequired[str]
  movementType: NotRequired[Literal['deposit', 'withdrawal']]
  amount: NotRequired[float]
  createdAt: NotRequired[datetime]
  walletId: NotRequired[str]
  currency: NotRequired[str]
  status: NotRequired[Literal['pending', 'completed', 'canceled']]
  serviceOrderName: NotRequired[Literal['wallet', 'admin', 'working-capital']]

adapter = TypeAdapter(list[Entry])

class FundingMovements(Endpoint):
  async def __call__(
    self,
    *,
    wallet_id: Any | None = None,
    currency: Any | None = None,
    from_: Any | None = None,
    to: Any | None = None,
    service_order_name: Any | None = None,
    movement_type: Any | None = None,
    limit: Any | None = None,
    validate: bool = True
  ) -> list[Entry]:
    """Get funding movements
    
    - `wallet_id`: Wallet id
    - `currency`: Currency
    - `from_`: The initial date with time
    - `to`: The end date with time
    - `service_order_name`: Service order name (issuer)
    - `movement_type`: Movement type
    - `limit`: The maximum number of movements returned
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/funding/GET/v1/trading/movement)"""
    params = {}
    if wallet_id is not None:
      params['walletId'] = wallet_id
    if currency is not None:
      params['currency'] = currency
    if from_ is not None:
      params['from'] = from_
    if to is not None:
      params['to'] = to
    if service_order_name is not None:
      params['serviceOrderName'] = service_order_name
    if movement_type is not None:
      params['movementType'] = movement_type
    if limit is not None:
      params['limit'] = limit
    r = await self.authed_request('GET', '/v1/trading/movement', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core import Endpoint

class Delete(Endpoint):
  async def delete(self, *, address_id: str, validate: bool = True):
    """Remove user address
    
    - `address_id`: The address id
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/DELETE/v1/account/address)"""
    params: dict = {
      'addressId': address_id,
    }
    r = await self.authed_request('DELETE', '/v1/account/address', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return r.json()



from datetime import datetime
from typing_extensions import Literal, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CurrencyOhlcResponse(TypedDict):
  open: str
  high: str
  low: str
  close: str
  average: str

adapter = TypeAdapter(CurrencyOhlcResponse)

class Ohlca(Endpoint):
  async def ohlca(
    self,
    symbol: str,
    *,
    time: datetime | None = None,
    timeframe: Literal['1H', '4H', '12H', '1D', '1W', '1M', '1Y'],
    validate: bool = True
  ) -> CurrencyOhlcResponse:
    """Returns OHLCA for the given symbol
    
    - `symbol`
    - `time`
    - `timeframe`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v1/currency/ohlca/{symbol})"""
    params: dict = {
      'timeframe': timeframe,
    }
    if time is not None:
      params['time'] = time
    r = await self.authed_request(
      'GET', f'/v1/currency/ohlca/{symbol}',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.types import EarnRewardsConfigResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(EarnRewardsConfigResponse)

class GetRewardsConfig(Endpoint):
  async def get_rewards_config(
    self,
    wallet_id: str,
    *,
    validate: bool = True
  ) -> EarnRewardsConfigResponse:
    """Retrieves rewards configuration for selected wallet
    
    - `wallet_id`: Wallet identifier
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/wallets/{walletId}/rewards/config)"""
    r = await self.authed_request(
      'GET', f'/v1/earn/wallets/{wallet_id}/rewards/config'
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal
from bit2me.types import EarnWalletResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(EarnWalletResponse)

class Wallets(Endpoint):
  async def wallets(
    self,
    *,
    user_currency: str | None = None,
    sort_by: Literal['currency', 'balance'] | None = None,
    sort_direction: Literal['ascending', 'descending'] | None = None,
    offset: int | None = None,
    limit: int | None = None,
    validate: bool = True
  ) -> EarnWalletResponse:
    """Get a list of earn wallets
    
    - `user_currency`: Currency to show convertedBalance
    - `sort_by`: Sorting field
    - `sort_direction`: Sorting direction
    - `offset`
    - `limit`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v2/earn/wallets)"""
    params = {}
    if user_currency is not None:
      params['userCurrency'] = user_currency
    if sort_by is not None:
      params['sortBy'] = sort_by
    if sort_direction is not None:
      params['sortDirection'] = sort_direction
    if offset is not None:
      params['offset'] = offset
    if limit is not None:
      params['limit'] = limit
    r = await self.authed_request('GET', '/v2/earn/wallets', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class DataItem(TypedDict):
  id: str
  minimum: int
  maximum: int

class DepositEstimationResponse(TypedDict):
  total: int
  data: list[DataItem]

adapter = TypeAdapter(DepositEstimationResponse)

class DepositEstimations(Endpoint):
  async def __call__(self, *, validate: bool = True) -> DepositEstimationResponse:
    """Return all deposit estimates.
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v1/account/deposit-estimation)"""
    r = await self.authed_request('GET', '/v1/account/deposit-estimation')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any, Mapping
from dataclasses import dataclass, field
import asyncio
import httpx

from ..exc import NetworkError

@dataclass
class HttpClient:
  lock: asyncio.Lock = field(default_factory=asyncio.Lock, init=False, repr=False)
  client_future: asyncio.Future[httpx.AsyncClient|None] = field(default_factory=asyncio.Future, init=False, repr=False)

  @property
  async def client(self) -> httpx.AsyncClient:
    if self.lock.locked() or self.client_future.done():
      if (client := await self.client_future) is not None:
        return client

    async with self.lock:
      client = await httpx.AsyncClient().__aenter__()
      self.client_future.set_result(client)
      return client

  async def __aenter__(self):
    await self.client

  async def __aexit__(self, exc_type, exc_value, traceback):
    client = await self.client
    if not self.lock.locked():
      async with self.lock:
        await client.__aexit__(exc_type, exc_value, traceback)
        self.client_future = asyncio.Future()

  async def request(
    self, method: str, url: str,
    *,
    content: httpx._types.RequestContent | None = None,
    data: httpx._types.RequestData | None = None,
    files: httpx._types.RequestFiles | None = None,
    json: Any | None = None,
    params: Mapping[str, Any] | None = None,
    headers: Mapping | None = None,
    cookies: httpx._types.CookieTypes | None = None,
    auth: httpx._types.AuthTypes | httpx._client.UseClientDefault | None = httpx.USE_CLIENT_DEFAULT,
    follow_redirects: bool | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    timeout: httpx._types.TimeoutTypes | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    extensions: httpx._types.RequestExtensions | None = None,
  ):
    try:
      client = await self.client
      return await client.request(
        method, url, params=params, cookies=cookies, json=json,
        content=content, data=data, files=files, auth=auth, follow_redirects=follow_redirects,
        timeout=timeout, extensions=extensions,
        headers=headers,
      )
    except httpx.HTTPError as e:
      req = f'{method} {url}'
      raise NetworkError(f'Error sending request to {req}', *e.args) from e

BIT2ME_API_URL = 'https://gateway.bit2me.com'

@dataclass
class HttpMixin:
  base_url: str = field(kw_only=True, default=BIT2ME_API_URL)
  http: HttpClient = field(kw_only=True, default_factory=HttpClient)

  async def __aenter__(self):
    await self.http.__aenter__()
    return self
  
  async def __aexit__(self, exc_type, exc_value, traceback):
    await self.http.__aexit__(exc_type, exc_value, traceback)

  async def request(
    self, method: str, path: str,
    *,
    content: httpx._types.RequestContent | None = None,
    data: httpx._types.RequestData | None = None,
    files: httpx._types.RequestFiles | None = None,
    json: Any | None = None,
    params: Mapping[str, Any] | None = None,
    headers: Mapping | None = None,
    cookies: httpx._types.CookieTypes | None = None,
    auth: httpx._types.AuthTypes | httpx._client.UseClientDefault | None = httpx.USE_CLIENT_DEFAULT,
    follow_redirects: bool | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    timeout: httpx._types.TimeoutTypes | httpx._client.UseClientDefault = httpx.USE_CLIENT_DEFAULT,
    extensions: httpx._types.RequestExtensions | None = None,
  ):
    return await self.http.request(
      method, self.base_url + path, params=params, headers=headers, cookies=cookies, json=json,
      content=content, data=data, files=files, auth=auth, follow_redirects=follow_redirects,
      timeout=timeout, extensions=extensions,
    )


from typing_extensions import Literal, AsyncIterable, TypeVar, Generic, Any, Callable, Awaitable
from abc import abstractmethod
from dataclasses import dataclass, field, replace
from collections import defaultdict
import inspect
import asyncio

from ..validation import TypedDict
from .base import RpcSocketClient

M = TypeVar('M', default=Any)
R = TypeVar('R', default=Any)
S = TypeVar('S', default=Any)
T = TypeVar('T', default=Any)
SP = TypeVar('SP', default=Any)
SR = TypeVar('SR', default=Any)
UR = TypeVar('UR', default=Any)

class Response(TypedDict, Generic[R]):
  kind: Literal['response']
  id: int | str
  response: R

class Subscription(TypedDict, Generic[S]):
  kind: Literal['subscription']
  channel: str
  data: S

Message = Response[R] | Subscription[S]

@dataclass
class Stream(AsyncIterable[S], Generic[S, SR, UR]):
  reply: SR
  stream: AsyncIterable[S]
  unsubscribe: Callable[[], Awaitable[UR]]

  def __aiter__(self):
    return self.stream.__aiter__()

  async def on(self, listener: Callable[[S], Awaitable[None] | None]):
    async for msg in self.stream:
      out = listener(msg)
      if inspect.isawaitable(out):
        await out

  def map(self, f: Callable[[S], T]) -> 'Stream[T, SR, UR]':
    async def stream() -> AsyncIterable[T]:
      async for msg in self.stream:
        yield f(msg)
    return replace(self, stream=stream()) # type: ignore

  def filter(self, f: Callable[[S], bool]) -> 'Stream[S, SR, UR]':
    async def stream() -> AsyncIterable[S]:
      async for msg in self.stream:
        if f(msg):
          yield msg
    return replace(self, stream=stream()) # type: ignore


@dataclass
class MultiplexStreamsRPCSocketClient(RpcSocketClient[M, R], Generic[M, R, S, SR, UR]):
  """Multiplexed request/response and streams socket client. It uses IDs to identify requests and responses. It also supports subscription to multiple channels."""
  replies: dict[int|str, asyncio.Future[R]] = field(default_factory=dict, init=False, repr=False)
  counter: int = field(default=0, init=False, repr=False)
  subscribers: dict[str, list[asyncio.Queue[S]]] = field(default_factory=lambda: defaultdict(list), init=False, repr=False)

  async def rpc_request(self, msg: M) -> R:
    id = self.counter
    self.counter += 1
    self.replies[id] = asyncio.Future()
    await self.rpc_send(id, msg)
    res = await self.wait_with_listener(self.replies[id])
    del self.replies[id]
    return res
    
  @abstractmethod
  async def req_subscription(self, channel: str, params: SP | None = None) -> SR:
    ...

  @abstractmethod
  async def req_unsubscription(self, channel: str, params: SP | None = None) -> UR:
    ...

  @abstractmethod
  def parse_msg(self, msg: str | bytes) -> Message[R, S] | None:
    ...

  def on_msg(self, msg: str | bytes):
    if (res := self.parse_msg(msg)) is not None:
      if res['kind'] == 'response':
        self.replies[res['id']].set_result(res['response']) # type: ignore
      else:
        for q in self.subscribers[res['channel']]:
          q.put_nowait(res['data']) # type: ignore

  @abstractmethod
  async def rpc_send(self, id: int, msg: M):
    ...

  async def subscribe(self, channel: str, params: SP | None = None) -> Stream[S, SR, UR]:
    q = asyncio.Queue()
    self.subscribers[channel].append(q)
    reply = await self.req_subscription(channel, params)

    async def stream() -> AsyncIterable[S]:
      while True:
        yield await self.wait_with_listener(asyncio.create_task(q.get()))

    async def unsubscribe() -> UR:
      del self.subscribers[channel]
      return await self.req_unsubscription(channel, params)

    return Stream(reply, stream(), unsubscribe)



from typing_extensions import Any, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Pocket(TypedDict):
  userId: str
  pocketId: str
  reference: str

class TellerPocketReferenceResponse(TypedDict):
  pocket: Pocket
  bankAccounts: dict[str, Any]

adapter = TypeAdapter(TellerPocketReferenceResponse)

class Pockets(Endpoint):
  async def __call__(
    self,
    *,
    pocket_id: str,
    validate: bool = True
  ) -> TellerPocketReferenceResponse:
    """Returns a reference for the specified pocket. Reference is generated if it doesn't exist.
    
    - `pocket_id`: Id of the pocket to retrieve the reference from
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/GET/v1/teller/pocket/reference)"""
    params: dict = {
      'pocketId': pocket_id,
    }
    r = await self.authed_request(
      'GET', '/v1/teller/pocket/reference',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(str)

class Subaccounts(Endpoint):
  async def subaccounts(
    self,
    *,
    identity_file_type: Literal['backDocument', 'frontDocument', 'selfie', 'livenessFrontDocument', 'livenessBackDocument', 'livenessSelfie'],
    sub_account_id: str,
    validate: bool = True
  ) -> str:
    """Download an identity file of a subaccount
    
    - `identity_file_type`: The identity file type to download
    - `sub_account_id`: The identifier of the subaccount
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/kyc/GET/v1/verifier/subaccount/file)"""
    params: dict = {
      'identityFileType': identity_file_type,
      'subAccountId': sub_account_id,
    }
    r = await self.authed_request(
      'GET', '/v1/verifier/subaccount/file',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.types import OrderResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(OrderResponse)

class Cancel(Endpoint):
  async def cancel(self, id: str, *, validate: bool = True) -> OrderResponse:
    """Cancel the order by identifier if this order is open or inactive.
    
    - `id`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/trading/DELETE/v1/trading/order/{id})"""
    r = await self.authed_request('DELETE', f'/v1/trading/order/{id}')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class LoanLtvResponse(TypedDict):
  guaranteeCurrency: NotRequired[str]
  guaranteeAmount: NotRequired[str]
  guaranteeAmountConverted: NotRequired[str]
  loanCurrency: NotRequired[str]
  loanAmount: NotRequired[str]
  loanAmountConverted: NotRequired[str]
  apr: NotRequired[str]
  ltv: NotRequired[str]
  userCurrency: NotRequired[str]

adapter = TypeAdapter(LoanLtvResponse)

class Ltv(Endpoint):
  async def __call__(
    self,
    *,
    guarantee_currency: str,
    guarantee_amount: str | None = None,
    loan_currency: str,
    loan_amount: str | None = None,
    user_currency: str,
    validate: bool = True
  ) -> LoanLtvResponse:
    """Get the calculated LTV (Loan To Value). You should provide one of (and only) loanAmount or guaranteeAmount
    
    - `guarantee_currency`
    - `guarantee_amount`
    - `loan_currency`
    - `loan_amount`
    - `user_currency`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/GET/v1/loan/ltv)"""
    params: dict = {
      'guaranteeCurrency': guarantee_currency,
      'loanCurrency': loan_currency,
      'userCurrency': user_currency,
    }
    if guarantee_amount is not None:
      params['guaranteeAmount'] = guarantee_amount
    if loan_amount is not None:
      params['loanAmount'] = loan_amount
    r = await self.authed_request('GET', '/v1/loan/ltv', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  iso: str
  iso3: NotRequired[str]
  name: str
  fips: str
  """Federal Information Processing Standard  (https://en.wikipedia.org/wiki/Federal_Information_Processing_Standards)"""

adapter = TypeAdapter(list[Entry])

class Countries(Endpoint):
  async def countries(
    self,
    country_isocode: str,
    *,
    lang_code: Literal['EN', 'ES'] | None = None,
    validate: bool = True
  ) -> list[Entry]:
    """Get country regions
    
    - `country_isocode`
    - `lang_code`: Language fo localize country names
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/misc/GET/v1/misc/country/{countryISOCode}/region)"""
    params = {}
    if lang_code is not None:
      params['langCode'] = lang_code
    r = await self.authed_request(
      'GET', f'/v1/misc/country/{country_isocode}/region',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Point(TypedDict):
  time: int
  interval: NotRequired[str]
  price: str

adapter = TypeAdapter(dict[str, list[Point]])

class Prices(Endpoint):
  async def prices(
    self,
    *,
    currency: str | None = None,
    interval: list[str] | None = None,
    validate: bool = True
  ) -> dict[str, list[Point]]:
    """Return cryptocurrency prices in the selected currency across the requested intervals, including the current rate.
    
    - `currency`
    - `interval`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v1/currency/prices)"""
    params = {}
    if currency is not None:
      params['currency'] = currency
    if interval is not None:
      params['interval'] = interval
    r = await self.authed_request('GET', '/v1/currency/prices', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class InitSubaccountTwoFactorRequest(TypedDict):
  subaccountUserId: str

class InitSubaccountTwoFactorResponse(TypedDict):
  secret: str

adapter = TypeAdapter(InitSubaccountTwoFactorResponse)

class EnableStart(Endpoint):
  async def enable_start(
    self,
    init_subaccount_two_factor_request: InitSubaccountTwoFactorRequest,
    *,
    validate: bool = True
  ) -> InitSubaccountTwoFactorResponse:
    """Start the Google Authenticator 2FA enablement flow for a subaccount.
    
    - `init_subaccount_two_factor_request`: The details to init subaccount 2fa process
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/POST/v1/account/subaccount/sfa)"""
    r = await self.authed_request(
      'POST', '/v1/account/subaccount/sfa',
      json=init_subaccount_two_factor_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Converted(TypedDict):
  amount: NotRequired[str]
  currency: NotRequired[str]

class TotalConverted(TypedDict):
  amount: float
  currency: str

class Entry(TypedDict):
  type: str
  currency: str
  total: str
  totalConverted: NotRequired[TotalConverted]
  converted: NotRequired[Converted]

adapter = TypeAdapter(list[Entry])

class GetSummaryByType(Endpoint):
  async def get_summary_by_type(
    self,
    type: Literal['deposit', 'reward', 'withdrawal'],
    *,
    user_currency: str | None = None,
    rate_moment: Literal['creation', 'current'],
    order: Literal['desc', 'asc'],
    validate: bool = True
  ) -> list[Entry]:
    """Retrieves summary of movements per operation (deposit, reward, withdrawal)
    
    - `type`: Operation type
    - `user_currency`: Currency to show balance
    - `rate_moment`: Rate moment
    - `order`: Sorting direction
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/movements/{type}/summary)"""
    params: dict = {
      'rateMoment': rate_moment,
      'order': order,
    }
    if user_currency is not None:
      params['userCurrency'] = user_currency
    r = await self.authed_request(
      'GET', f'/v1/earn/movements/{type}/summary',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class UpdateTransactionRequest(TypedDict):
  note: str

adapter = TypeAdapter(BooleanResultResponse)

class Update(Endpoint):
  async def update(
    self,
    id: str,
    update_transaction_request: UpdateTransactionRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Updates some data of the specified transaction
    
    - `id`: The transaction id
    - `update_transaction_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/PUT/v1/wallet/transaction/{id})"""
    r = await self.authed_request(
      'put', f'/v1/wallet/transaction/{id}',
      json=update_transaction_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any

class Error(Exception):
  def __str__(self):
    args = self.args[0] if len(self.args) == 1 else ', '.join(self.args)
    return f'{self.__class__.__name__}({args})'

class NetworkError(Error):
  """Error accessing the API"""
  def __str__(self):
    return super().__str__()

class ValidationError(Error):
  """The response shape did not match the expected schema"""
  def __str__(self):
    return super().__str__()

class ApiError(Error):
  """The remote API returned a non 2xx status code"""
  def __init__(self, status: int, payload: Any, *args):
    self.status = status
    self.payload = payload
    super().__init__(status, payload, *args)

  def __str__(self):
    return f'ApiError({self.status}), {self.payload})'

  @staticmethod
  def of(status: int, payload: Any, *args):
    if status == 400:
      cls = BadRequest
    elif status == 401:
      cls = Unauthorized
    elif status == 404:
      cls = NotFound
    elif status == 429:
      cls = RateLimited
    elif status == 404:
      cls = NotFound
    elif 500 <= status < 600:
      cls = InternalServerError
    else:
      cls = ApiError
    return cls(status, payload, *args)

class RateLimited(ApiError):
  """The remote API returned a 429 status code"""
  def __str__(self):
    return super().__str__()

class BadRequest(ApiError):
  """The remote API returned a 400 status code"""
  def __str__(self):
    return super().__str__()

class Unauthorized(ApiError):
  """The remote API returned a 401 status code"""
  def __str__(self):
    return super().__str__()

class NotFound(ApiError):
  """The remote API returned a 404 status code"""
  def __str__(self):
    return super().__str__()

class InternalServerError(ApiError):
  """The remote API returned a 5xx status code"""
  def __str__(self):
    return super().__str__()


from typing_extensions import NotRequired, TypedDict
from bit2me.types import UserAddress
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class UserAddressBody(TypedDict):
  id: NotRequired[str]
  userId: NotRequired[str]
  alias: NotRequired[str]
  streetAddress: str
  city: str
  stateCode: str
  """Value obtained from 'fips' (Federal Information Processing Standard  (https://en.wikipedia.org/wiki/Federal_Information_Processing_Standards)) field in /v1/misc/country/{countryISOCode}/region response.

  Additional information of this endpoint is available in misc section"""
  zip: str
  countryCode: str
  isResidence: NotRequired[bool]
  isDefaultAddress: NotRequired[bool]
  nationalityCountryCode: str

adapter = TypeAdapter(UserAddress)

class Create(Endpoint):
  async def create(
    self,
    user_address_body: UserAddressBody,
    *,
    validate: bool = True
  ) -> UserAddress:
    """Add address to an user
    
    - `user_address_body`: Address params
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/POST/v2/account/address)"""
    r = await self.authed_request(
      'POST', '/v2/account/address',
      json=user_address_body
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core.mixin import Router
from .pockets import Pockets
from .settings import Settings
from .transactions import Transactions

class Wallet(Router):
  pockets: Pockets
  settings: Settings
  transactions: Transactions



from typing_extensions import NotRequired, TypedDict
from bit2me.types import MillisTimestamp
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  timestamp: NotRequired[MillisTimestamp]
  symbol: NotRequired[str]
  """Market symbol"""
  open: NotRequired[float]
  """Opening price (price 24 horus ago)"""
  bid: NotRequired[float]
  """Highest price a buyer will pay for order"""
  ask: NotRequired[float]
  """Lowest price a seller will take for order"""
  close: NotRequired[float]
  """Closing price (last trade price)"""
  high: NotRequired[float]
  """Highest price in the last 24 hours"""
  low: NotRequired[float]
  """Lowest price in the last 24 hours"""
  percentage: NotRequired[float]
  """Percentage of current price versus opening price"""
  baseVolume: NotRequired[float]
  """Volume traded in terms of the base currency"""
  quoteVolume: NotRequired[float]
  """Volume traded in terms of the quote currency"""

adapter = TypeAdapter(list[Entry])

class Tickers(Endpoint):
  async def tickers(self, *, symbol: str | None = None, validate: bool = True) -> list[Entry]:
    """Get ticker information (OHLCV, current best bid and ask, percentage versus price 24 hours ago) for all markets or by requested market symbol. The data refers to the last 24 hours from the date indicated by the timestamp.
    
    - `symbol`: Market symbol (optional, default all markets)
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/marketdata/GET/v2/trading/tickers)"""
    params = {}
    if symbol is not None:
      params['symbol'] = symbol
    r = await self.request('GET', '/v2/trading/tickers', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class RewardAllowedItem(TypedDict):
  currency: NotRequired[str]
  """Currency where can receive rewards"""
  extraYield: NotRequired[float]
  """Extra Yield by currency applied in the reward"""
  type: NotRequired[Literal['daily', 'weekly', 'monthly']]
  """Reward type"""

class Entry(TypedDict):
  currency: NotRequired[str]
  """Currency symbol, it should be an allowed currency"""
  disabled: NotRequired[bool]
  """Currency status on earn"""
  isNew: NotRequired[bool]
  """Currency added on earn in the last 7 days"""
  currenciesRewardAllowed: NotRequired[list[RewardAllowedItem]]
  """Currency rewards allowed with their info"""
  levelExtraYieldType: NotRequired[Literal['space-pool', 'provided']]

adapter = TypeAdapter(list[Entry])

class Assets(Endpoint):
  async def assets(
    self,
    *,
    type: Literal['partner', 'farming-pool'] | None = None,
    validate: bool = True
  ) -> list[Entry]:
    """Retrieves full list of supported assets in earn service
    
    - `type`: Asset type
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v2/earn/assets)"""
    params = {}
    if type is not None:
      params['type'] = type
    r = await self.request('GET', '/v2/earn/assets', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import (
  OrderResponse,
  OrderSide,
  OrderType,
  PostOnlyParam,
  TimeInForce
)
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TradingOrdersCreateRequest(TypedDict):
  side: OrderSide
  symbol: str
  """The market symbol"""
  price: NotRequired[str]
  stopPrice: NotRequired[str]
  amount: str
  orderType: OrderType
  clientOrderId: NotRequired[str]
  """Optional client internal order identifier"""
  postOnly: NotRequired[PostOnlyParam]
  timeInForce: NotRequired[TimeInForce]
  amountInQuote: NotRequired[bool]
  """Option to express volume in quote currency. This option is supported only for market orders. When using this parameter, Bit2Me calculates market price of the asset and immediately executes the order. This means that 100% precision is not guaranteed. When using this feature, you acknowledge that you understand how this works and accept that the differences arising from this method are not an error and are not Bit2Me responsibility to fix or provide you with additional information."""

adapter = TypeAdapter(OrderResponse)

class Create(Endpoint):
  async def create(
    self,
    trading_orders_create_request: TradingOrdersCreateRequest,
    *,
    validate: bool = True
  ) -> OrderResponse:
    """Place a new order.
    
    See `GET /v1/trading/market-config` for the available trading assets, their price and quantity precisions, and order minimums.
    
    - `trading_orders_create_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/trading/POST/v1/trading/order)"""
    r = await self.authed_request(
      'POST', '/v1/trading/order',
      json=trading_orders_create_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from bit2me.types import PointsParam
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class ExtraMeasuresNotApproved(TypedDict):
  compliance: list[Literal['MR2']]

class KycExpress(TypedDict):
  status: NotRequired[Literal['success', 'rejected', 'uncompleted']]
  updatedAt: NotRequired[str]

class Risk(TypedDict):
  level: Literal['undefined', 'low', 'medium', 'high', 'unacceptable']
  minPoints: NotRequired[PointsParam]
  maxPoints: NotRequired[PointsParam]
  action: NotRequired[str]

class IdentityVerificationResponse(TypedDict):
  status: Literal['nodata', 'pending', 'verified', 'error']
  error: NotRequired[list[Literal['number', 'expiryDate', 'noDocUploaded', 'imageDoc', 'imageSelfie', 'other', 'unacceptableRisk', 'underAge', 'fraud', 'noNationality', 'idCountryDiffersResidenceCountry', 'kycUpdateRequired']]]
  image: list[Literal['document', 'selfie']]
  disableProcess: bool
  risk: NotRequired[Risk]
  kycExpress: NotRequired[KycExpress]
  questions: NotRequired[bool]
  extraMeasurePending: NotRequired[bool]
  extraMeasuresNotApproved: NotRequired[ExtraMeasuresNotApproved]
  tier: NotRequired[float]

adapter = TypeAdapter(IdentityVerificationResponse)

class GetStatus(Endpoint):
  async def get_status(self, *, validate: bool = True) -> IdentityVerificationResponse:
    """Return the current status of the identity verification
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v1/account/verify/identity)"""
    r = await self.authed_request('GET', '/v1/account/verify/identity')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import MillisTimestamp
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TradingOrderBookResponse(TypedDict):
  asks: NotRequired[list[list[float]]]
  """Entries containing sell orders with price and volume"""
  bids: NotRequired[list[list[float]]]
  """Entries containing buy orders with price and volume"""
  timestamp: NotRequired[MillisTimestamp]
  symbol: NotRequired[str]
  """Market symbol"""

adapter = TypeAdapter(TradingOrderBookResponse)

class OrderBook(Endpoint):
  async def order_book(
    self,
    *,
    symbol: str,
    validate: bool = True
  ) -> TradingOrderBookResponse:
    """Get the order book associated to the market symbol.
    
    - `symbol`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/marketdata/GET/v2/trading/order-book)"""
    params: dict = {
      'symbol': symbol,
    }
    r = await self.request('GET', '/v2/trading/order-book', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TellerOrderStatusResponse(TypedDict):
  status: NotRequired[str]
  """The status of the order (accepted or pending)"""

adapter = TypeAdapter(TellerOrderStatusResponse)

class GetStatus(Endpoint):
  async def get_status(self, *, id: str, validate: bool = True) -> TellerOrderStatusResponse:
    """Gets order status. If no order is found, "pending" status is returned as default value.
    
    - `id`: The order ID or the wallet proforma ID
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/GET/v1/teller/order/status)"""
    params: dict = {
      'id': id,
    }
    r = await self.authed_request('GET', '/v1/teller/order/status', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypeVar, Any, Generic
from abc import ABC, abstractmethod
import asyncio
from dataclasses import dataclass, field
from datetime import timedelta
import logging
import websockets

from ..exc import NetworkError

T = TypeVar('T')
M = TypeVar('M', default=Any)
R = TypeVar('R', default=Any)

logger = logging.getLogger('core.ws')

@dataclass
class Context:
  ws: websockets.ClientConnection
  listener: asyncio.Task

@dataclass(kw_only=True)
class SocketClient(ABC):
  url: str
  timeout: timedelta = timedelta(seconds=10)
  ctx_future: asyncio.Future[Context] = field(default_factory=asyncio.Future, init=False)
  open_lock: asyncio.Lock = field(default_factory=asyncio.Lock, init=False)
  close_lock: asyncio.Lock = field(default_factory=asyncio.Lock, init=False)
  
  @property
  async def ctx(self) -> Context:
    return await self.open()
  
  @property
  async def ws(self) -> websockets.ClientConnection:
    return (await self.ctx).ws
  
  async def __aenter__(self):
    await self.open()
    return self
  
  async def __aexit__(self, exc_type, exc_value, traceback):
    await self.close(await self.ctx, exc_type, exc_value, traceback)

  async def force_open(self):
    logger.info('Connecting...')
    async def connect():
      try:
        return await websockets.connect(self.url, open_timeout=self.timeout.total_seconds())
      except websockets.exceptions.WebSocketException as e:
        raise NetworkError(f'Failed to connect to {self.url}', *e.args) from e
      
    ws = await connect()
    logger.info('Connected!')
    return Context(
      ws=ws,
      listener=asyncio.create_task(self.listener(ws)),
    )
  
  async def open(self) -> Context:
    if self.open_lock.locked() or self.ctx_future.done():
      return await self.ctx_future

    async with self.open_lock:
      ctx = await self.force_open()
      self.ctx_future.set_result(ctx)
      return ctx

  async def force_close(self, ctx: Context, exc_type=None, exc_value=None, traceback=None):
    ctx.listener.cancel()
    await ctx.ws.__aexit__(exc_type, exc_value, traceback)

  async def close(self, ctx: Context, exc_type=None, exc_value=None, traceback=None):
    if not self.close_lock.locked():
      async with self.close_lock:
        await self.force_close(ctx, exc_type, exc_value, traceback)

  async def listener(self, ws: websockets.ClientConnection, /):
    while True:
      try:
        msg = await ws.recv()
        logger.debug('Received: %s', msg)
        self.on_msg(msg)
      except websockets.exceptions.WebSocketException as e:
        logger.error('Error receiving message: %s', e)
        raise NetworkError('Error receiving message', *e.args) from e

  @abstractmethod
  def on_msg(self, msg: str | bytes):
    ...

  async def wait_with_listener(self, fut: asyncio.Future[T]) -> T:
    """Wait for a future to complete, propagating any exceptions if the listener task fails or gets cancelled"""
    ctx = await self.ctx
    async def coro():
      return await fut
    task = asyncio.create_task(coro())
    done, _ = await asyncio.wait([task, ctx.listener], return_when='FIRST_COMPLETED')
    if ctx.listener in done:
      if (exc := ctx.listener.exception()) is not None:
        raise exc
      else:
        raise asyncio.CancelledError('Listener task got cancelled')
    return task.result()

class RpcSocketClient(SocketClient, Generic[M, R]):
  """Base request/response socket client."""
  @abstractmethod
  async def rpc_request(self, msg: M) -> R:
    ...



from datetime import datetime
from typing_extensions import Any, Literal, NotRequired, TypedDict
from bit2me.types import (
  AmountCurrencyObject,
  Phone,
  Rate,
  TransactionSubsFeeTypeParam
)
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class BankAccount(TypedDict):
  bankAccount: str
  swiftBic: NotRequired[str]
  country: str
  receiverName: NotRequired[str]
  bankName: NotRequired[str]
  bankAddress: NotRequired[str]
  bankCode: NotRequired[str]
  bankBranch: NotRequired[str]

class Creditcard(TypedDict):
  cardId: str

class Fixed(TypedDict):
  amount: str
  currency: str
  """Valid currency symbol"""

class Flip(TypedDict):
  """Flip fee"""
  percentage: str
  amount: str
  currency: str
  """Valid currency symbol"""

class Network(TypedDict):
  """Network fee"""
  amount: str
  currency: str

class UserAmount(TypedDict):
  currency: str
  """Valid currency symbol"""
  amount: str

class Variable(TypedDict):
  percentage: str
  amount: str
  currency: str
  """Valid currency symbol"""

class Destination(TypedDict):
  address: NotRequired[str]
  pocket: NotRequired[str]
  bankAccount: NotRequired[BankAccount]
  email: NotRequired[str]
  phone: NotRequired[Phone]
  alias: NotRequired[str]
  network: NotRequired[str]
  """The destination address network"""

class Origin(TypedDict):
  creditcard: NotRequired[Creditcard]

class Teller(TypedDict):
  fixed: Fixed
  variable: Variable

class Fee(TypedDict):
  """Different fees for the transaction"""
  network: NotRequired[Network]
  flip: NotRequired[Flip]
  teller: NotRequired[Teller]

class WalletTransactionProformaRequest(TypedDict):
  operation: NotRequired[Literal['purchase', 'withdrawal-trading', 'deposit-trading', 'social-payment', 'launchpad-purchase', 'buy', 'sell']]
  """This property is not necessary for other operation types"""
  pair: NotRequired[str]
  pocket: NotRequired[str]
  amount: str
  currency: str
  type: NotRequired[TransactionSubsFeeTypeParam]
  concept: NotRequired[str]
  note: NotRequired[str]
  receiverName: NotRequired[str]
  origin: NotRequired[Origin]
  destination: NotRequired[Destination]
  userCurrency: NotRequired[str]
  """The user's currency (used to show a rate from it to Euro)"""
  queryParams: NotRequired[dict[str, Any]]
  """Used for some cryptos that need extra parameters to do blockchain sendings. For example, memo in XRP."""

class WalletTransactionProformaResponse(TypedDict):
  id: str
  expirationTime: datetime
  origin: AmountCurrencyObject
  destination: AmountCurrencyObject
  fee: NotRequired[Fee]
  flip: NotRequired[Rate]
  userRate: NotRequired[Rate]
  userAmount: NotRequired[UserAmount]

adapter = TypeAdapter(WalletTransactionProformaResponse)

class Preview(Endpoint):
  async def preview(
    self,
    wallet_transaction_proforma_request: WalletTransactionProformaRequest,
    *,
    validate: bool = True
  ) -> WalletTransactionProformaResponse:
    """Create a new proforma transaction, including its expiration time.
    
    - If `type` is not specified, `REA` is used by default.
    - `destination` only accepts one field. Use `destination.address` for a cryptocurrency address or `destination.pocket` for the destination pocket ID.
    - The user's email must be validated before calling this endpoint.
    - For peer-to-peer transactions, a sell operation must be made on the source pocket and a buy operation must be made on the destination pocket.
    - All blockchain withdrawals must include Travel Rule information before they can be processed. See `POST /v1/blockchain-manager/travel-rule-order/{orderId}`.
    
    - `wallet_transaction_proforma_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/POST/v1/wallet/transaction/proforma)"""
    r = await self.authed_request(
      'POST', '/v1/wallet/transaction/proforma',
      json=wallet_transaction_proforma_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class IdentityVerificationRetryEligibilityResponse(TypedDict):
  result: bool

adapter = TypeAdapter(IdentityVerificationRetryEligibilityResponse)

class GetRetryWindow(Endpoint):
  async def get_retry_window(
    self,
    *,
    validate: bool = True
  ) -> IdentityVerificationRetryEligibilityResponse:
    """Gets a response based on whether the user can retry the verification process
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/kyc/GET/v1/account/verify/identity/verification/retry)"""
    r = await self.authed_request(
      'GET', '/v1/account/verify/identity/verification/retry'
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Any, Literal, NotRequired, TypedDict

class BooleanResultResponse(TypedDict):
  result: bool

class CreatedResourceIdResponse(TypedDict):
  id: str
  """Identifier of the created resource (e.g. executed wallet transaction)"""

class CustomerRequestTransaction(TypedDict):
  userId: str
  amount: str
  pair: Literal['BTC/EUR', 'B2M/EUR', '...']
  createdAt: datetime

class DepositEstimation(TypedDict):
  minimum: int
  maximum: int

class EarnRewardsConfigResponse(TypedDict):
  walletId: NotRequired[str]
  userId: NotRequired[str]
  currency: NotRequired[str]
  rewardCurrency: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]

class EarnWalletRecordConvertedBalance(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]

class FundsOrigin(TypedDict):
  type: Literal['salary', 'incomes', 'investments', 'pensions', 'donations', 'propertiesSell', 'others']
  othersReason: NotRequired[str]

class ItemConvertedBalance(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]

class LoanAction(TypedDict):
  orderId: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  userId: NotRequired[str]
  requestedByUserId: NotRequired[str]
  guaranteeCurrency: NotRequired[str]
  guaranteeOriginalAmount: NotRequired[str]
  guaranteeAmount: NotRequired[str]
  loanCurrency: NotRequired[str]
  loanOriginalAmount: NotRequired[str]
  loanAmount: NotRequired[str]
  paybackAmount: NotRequired[str]
  ltv: NotRequired[str]
  apr: NotRequired[str]
  status: NotRequired[str]
  startedAt: NotRequired[datetime]
  finishedAt: NotRequired[datetime]
  expiresAt: NotRequired[datetime]
  paybackPrincipalAmount: NotRequired[str]
  paybackInterestAmount: NotRequired[str]
  interestAmount: NotRequired[str]

class LockPeriod(TypedDict):
  lockPeriodId: NotRequired[str]
  months: NotRequired[int]

MillisTimestamp = int

OffsetParam = int

class OrderResponse(TypedDict):
  id: NotRequired[str]
  """Order identifier"""
  userId: NotRequired[str]
  """User identifier"""
  side: NotRequired[Literal['buy', 'sell']]
  """Order direction"""
  symbol: NotRequired[str]
  """Market symbol"""
  fromCurrency: NotRequired[str]
  """Base currency"""
  toCurrency: NotRequired[str]
  """Quote currency"""
  price: NotRequired[str | float]
  stopPrice: NotRequired[str | float]
  amount: NotRequired[str]
  orderAmount: NotRequired[str | float]
  filledAmount: NotRequired[float]
  """Filled order size in terms of base currency"""
  dustAmount: NotRequired[float]
  """Difference amount between the order amount and the filled amount when the order status is filled"""
  feeAmount: NotRequired[float]
  """Fee percentage respect to trade amount `amount`"""
  feeCurrency: NotRequired[str | None]
  """Field `feeAmount` currency"""
  status: NotRequired[Literal['open', 'filled', 'cancelled', 'inactive']]
  """Order status. The stop limit order remains inactive until the stop price is reached, then it becomes open or filled"""
  orderType: NotRequired[Literal['limit', 'stop-limit', 'market']]
  """Order type"""
  cost: NotRequired[float]
  """Total cost by multiplying amount by price"""
  createdAt: NotRequired[datetime]
  """Date time in ISO 8601 string format"""
  updatedAt: NotRequired[datetime]
  """Date time in ISO 8601 string format"""
  cancelReason: NotRequired[str | None]
  """Detailed message about order cancel reason"""
  clientOrderId: NotRequired[str | None]
  """Client order internal identifier"""
  postOnly: NotRequired[bool | None]
  timeInForce: NotRequired[Literal['GTC', 'IOC', 'FOK']]
  """Time-in-force of the order to specify how long it should remain in the order book before being cancelled:
GTC (Good-'til-cancelled) is default if the parameter is omitted.
IOC (Immediate-or-cancel) will immediately execute the amount possible and cancel any remaining balance rather than resting in the book.
FOK (Fill-or-kill) will immediately wait for order to be fully executed without taking the risk of receiving partial fills"""

OrderSide = Literal['buy', 'sell']

OrderStatus = Literal['open', 'filled', 'cancelled', 'inactive']

OrderType = Literal['limit', 'stop-limit', 'market']

class Pair(TypedDict):
  base: str
  """Valid currency symbol"""
  quote: str
  """Valid currency symbol"""

PayOrderStatus = Literal['waiting-funds', 'pending', 'accepted', 'cancelled', 'expired', 'error']

PayOrderType = Literal['email', 'phone', 'alias']

class Phone(TypedDict):
  number: str
  countryCode: str

PocketColor = int

PointsParam = int

PostOnlyParam = bool

SortDirectionParam = Literal['asc', 'desc']

class StatusChangesItem(TypedDict):
  newStatus: NotRequired[Literal['waiting-funds', 'pending', 'accepted', 'cancelled', 'expired', 'error']]
  author: NotRequired[str]
  created: NotRequired[datetime]
  reason: NotRequired[str]

TimeInForce = Literal['GTC', 'IOC', 'FOK']

class TokenResponse(TypedDict):
  token: str
  expirationTime: int

class TradeResponse(TypedDict):
  id: NotRequired[str]
  """Trade identifier"""
  orderId: NotRequired[str]
  """Order identifier"""
  symbol: NotRequired[str]
  """Market symbol"""
  side: NotRequired[Literal['buy', 'sell']]
  """Order direction"""
  orderType: NotRequired[Literal['limit', 'stop-limit', 'market']]
  """Order type"""
  price: NotRequired[str | float]
  """Spot API may return numeric or string decimals"""
  amount: NotRequired[str | float]
  """Spot API may return numeric or string decimals"""
  priceCurrency: NotRequired[str]
  """Field `price` currency"""
  amountCurrency: NotRequired[str]
  """Field `amount` currency"""
  isMaker: NotRequired[bool]
  """`true` if trade was executed with user as the maker, `false` if taker"""
  createdAt: NotRequired[datetime]
  """Date time in ISO 8601 string format"""
  cost: NotRequired[float]
  """Total cost (quote currency)"""
  costEuro: NotRequired[float]
  """Total cost (EURO currency)"""
  clientOrderId: NotRequired[str | None]
  """Client order internal identifier (null when not set)"""
  feeAmount: NotRequired[float]
  """Fee amount (currency indicated by `feeCurrency` field)"""
  feePercentage: NotRequired[float]
  """Fee percentage respect to trade amount `amount`"""
  feeCurrency: NotRequired[str]
  """Field `feeAmount` currency"""

TransactionSubsFeeTypeParam = Literal['SEA', 'REA']

class UserAddress(TypedDict):
  id: NotRequired[str]
  userId: NotRequired[str]
  alias: NotRequired[str]
  streetAddress: str
  city: str
  stateCode: str
  """Value obtained from 'fips' (Federal Information Processing Standard  (https://en.wikipedia.org/wiki/Federal_Information_Processing_Standards)) field in /v1/misc/country/{countryISOCode}/region response.

  Additional information of this endpoint is available in misc section"""
  zip: str
  countryCode: str
  isResidence: NotRequired[bool]
  isDefaultAddress: NotRequired[bool]

UserType = Literal['person', 'company', 'employee']

class WalletResponse(TypedDict):
  id: NotRequired[str]
  """Wallet identifier"""
  userId: NotRequired[str]
  """User identifier"""
  currency: NotRequired[str]
  """Valid currency symbol"""
  balance: NotRequired[float]
  """Balance available"""
  blockedBalance: NotRequired[float]
  """Balance in use on active orders"""
  createdAt: NotRequired[datetime]
  """Date time in ISO 8601 string format"""

class DataItem(TypedDict):
  walletId: NotRequired[str]
  userId: NotRequired[str]
  currency: NotRequired[str]
  balance: NotRequired[str]
  lockPeriod: NotRequired[LockPeriod]
  convertedBalance: NotRequired[ItemConvertedBalance]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]

class EarnWalletRecord(TypedDict):
  """Single Earn wallet returned by GET /v1/earn/wallets/{walletId}"""
  walletId: NotRequired[str]
  userId: NotRequired[str]
  currency: NotRequired[str]
  balance: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  convertedBalance: NotRequired[EarnWalletRecordConvertedBalance]
  lockPeriod: NotRequired[dict[str, Any]]

class PayOrderData(TypedDict):
  orderId: str
  createdAt: datetime
  expiryDate: datetime
  userId: str
  pocketId: str
  type: Literal['email', 'phone', 'alias']
  email: NotRequired[str]
  phone: NotRequired[Phone]
  currency: str
  amount: str
  status: Literal['waiting-funds', 'pending', 'accepted', 'cancelled', 'expired', 'error']
  statusChanges: list[StatusChangesItem]
  senderName: str
  alias: NotRequired[str]
  note: NotRequired[str]
  walletMovementId: str

class RateRate(TypedDict):
  value: str
  pair: Pair

class EarnWalletResponse(TypedDict):
  total: NotRequired[int]
  data: NotRequired[list[DataItem]]

class Rate(TypedDict):
  rate: NotRequired[RateRate]

class AmountCurrencyObject(TypedDict):
  amount: str
  currency: str
  """Valid currency symbol"""
  rate: NotRequired[Rate]



from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[str])

class Conversion(Endpoint):
  async def conversion(
    self,
    *,
    from_: str,
    to: str,
    value: str,
    time: str,
    validate: bool = True
  ) -> list[str]:
    """Convert the specified amounts to the specified currency
    
    - `from_`
    - `to`
    - `value`
    - `time`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v1/currency/convert)"""
    params: dict = {
      'from': from_,
      'to': to,
      'value': value,
      'time': time,
    }
    r = await self.authed_request('GET', '/v1/currency/convert', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.core import Endpoint

class TellerOrderCancelRequest(TypedDict):
  orderId: str
  description: str

class Cancel(Endpoint):
  async def cancel(
    self,
    teller_order_cancel_request: TellerOrderCancelRequest,
    *,
    validate: bool = True
  ):
    """Changes an order status from "waiting-user" to "cancelled" after the user accepting the transaction
    
    - `teller_order_cancel_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/POST/v1/teller/order/cancel)"""
    r = await self.authed_request(
      'POST', '/v1/teller/order/cancel',
      json=teller_order_cancel_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CreatePocketRequest(TypedDict):
  currency: str
  """Valid currency symbol"""
  name: str

class CreatePocketResponse(TypedDict):
  id: str

adapter = TypeAdapter(CreatePocketResponse)

class Create(Endpoint):
  async def create(
    self,
    create_pocket_request: CreatePocketRequest,
    *,
    validate: bool = True
  ) -> CreatePocketResponse:
    """Create a new pocket
    
    - `create_pocket_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/POST/v1/wallet/pocket)"""
    r = await self.authed_request(
      'POST', '/v1/wallet/pocket',
      json=create_pocket_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CreateLoanRequest(TypedDict):
  guaranteeCurrency: str
  guaranteeAmount: str
  loanCurrency: str
  loanAmount: str

class CreateLoanResponse(TypedDict):
  orderId: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  userId: NotRequired[str]
  requestedByUserId: NotRequired[str]
  guaranteeCurrency: NotRequired[str]
  guaranteeOriginalAmount: NotRequired[str]
  guaranteeAmount: NotRequired[str]
  loanCurrency: NotRequired[str]
  loanOriginalAmount: NotRequired[str]
  loanAmount: NotRequired[str]
  paybackAmount: NotRequired[str]
  ltv: NotRequired[str]
  apr: NotRequired[str]
  status: NotRequired[str]
  startedAt: NotRequired[datetime]
  finishedAt: NotRequired[datetime]
  expiresAt: NotRequired[datetime]
  paybackPrincipalAmount: NotRequired[str]
  paybackInterestAmount: NotRequired[str]
  interestAmount: NotRequired[str]

adapter = TypeAdapter(CreateLoanResponse)

class Create(Endpoint):
  async def create(
    self,
    create_loan_request: CreateLoanRequest,
    *,
    validate: bool = True
  ) -> CreateLoanResponse:
    """Create new loan
    
    - `create_loan_request`: data
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/POST/v1/loan/orders)"""
    r = await self.authed_request(
      'POST', '/v1/loan/orders',
      json=create_loan_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class SignInTwoFactorV3response(TypedDict):
  sfaType: Literal['gauth', 'email', 'sms', 'call', 'whatsapp']
  retryIntervalSeconds: NotRequired[float]

adapter = TypeAdapter(SignInTwoFactorV3response)

class TwoFactor(Endpoint):
  async def two_factor(self, *, validate: bool = True) -> SignInTwoFactorV3response:
    """Get the active user verification method and initialize the verification process.
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/access/GET/v3/signin/sfa)"""
    r = await self.authed_request('GET', '/v3/signin/sfa')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  id: NotRequired[str]
  """Market identifier"""
  symbol: NotRequired[str]
  """Market symbol"""
  minAmount: NotRequired[float]
  """Minimum order amount (in terms of base currency)"""
  maxAmount: NotRequired[float]
  """Maximum order amount (in terms of base currency)"""
  minPrice: NotRequired[float]
  """Minimum order price (in terms of quote currency)"""
  maxPrice: NotRequired[float]
  """Maximum order price (in terms of quote currency)"""
  minOrderSize: NotRequired[float]
  """Minimum order size (in terms of base amount per quote price)"""
  pricePrecision: NotRequired[float]
  """Scaling decimals places for price"""
  tickSize: NotRequired[float]
  """Decimal number representing scaling decimals places for price"""
  amountPrecision: NotRequired[float]
  """Scaling decimals places for amount"""
  marketEnabled: NotRequired[Literal['enabled', 'enabled_at', 'frozen', 'disabled']]
  """The current status of the market. The market can be enabled, disabled, enabled at specified date in the `marketEnabledAt` field, or frozen, which does not allow orders to be added or deleted"""
  marketEnabledAt: NotRequired[datetime | None]
  """Date time in ISO 8601 string format"""
  initialPrice: NotRequired[float]
  """Initial market price. If the market is not yet enabled, the orders must take that reference price to be placed above or below"""

adapter = TypeAdapter(list[Entry])

class Markets(Endpoint):
  async def __call__(
    self,
    *,
    symbol: str | None = None,
    validate: bool = True
  ) -> list[Entry]:
    """Get a list of markets (quantity and price precisions, order minimums and maximums, status).
    
    - `symbol`: The market symbol to filter (optional, by default returns all markets)
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/marketdata/GET/v1/trading/market-config)"""
    params = {}
    if symbol is not None:
      params['symbol'] = symbol
    r = await self.request('GET', '/v1/trading/market-config', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core.mixin import Router
from .addresses import Addresses
from .delete import Delete
from .deposit_estimations import DepositEstimations
from .employment_statuses import EmploymentStatuses
from .funds_origins import FundsOrigins
from .identity_verification import IdentityVerification
from .person_identity import PersonIdentity
from .professions import Professions
from .purposes import Purposes
from .settings import Settings
from .subaccounts import Subaccounts
from .update import Update

class Account(Router):
  addresses: Addresses
  delete: Delete
  deposit_estimations: DepositEstimations
  employment_statuses: EmploymentStatuses
  funds_origins: FundsOrigins
  identity_verification: IdentityVerification
  person_identity: PersonIdentity
  professions: Professions
  purposes: Purposes
  settings: Settings
  subaccounts: Subaccounts
  update: Update



from bit2me.core.mixin import Router
from .balance import Balance
from .candles import Candles
from .funding_movements import FundingMovements
from .markets import Markets
from .orders import Orders
from .trades import Trades
from .wallets import Wallets

class Trading(Router):
  balance: Balance
  candles: Candles
  funding_movements: FundingMovements
  markets: Markets
  orders: Orders
  trades: Trades
  wallets: Wallets



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class SettingsEnableTwoFactorStartResponse(TypedDict):
  qrImage: str
  """QR image containing the otpauth URL for Google Authenticator (e.g. data:image/png;base64,iVB...YII=)"""
  secret: str
  """Shared secret key (base-32 encoded)"""
  name: str
  """The name to use with Google Authenticator"""

adapter = TypeAdapter(SettingsEnableTwoFactorStartResponse)

class EnableStart(Endpoint):
  async def enable_start(
    self,
    *,
    validate: bool = True
  ) -> SettingsEnableTwoFactorStartResponse:
    """Start the 2FA enablement flow. After calling this endpoint, call `PUT /v1/account/settings/sfa` with a TOTP to finish the process.
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/POST/v1/account/settings/sfa)"""
    r = await self.authed_request('POST', '/v1/account/settings/sfa')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import Phone
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class DataItem(TypedDict):
  id: NotRequired[str]
  email: NotRequired[str]
  name: NotRequired[str]
  surname: NotRequired[str]
  phone: NotRequired[Phone]

class ListSubaccountsResponse(TypedDict):
  total: NotRequired[float]
  data: NotRequired[list[DataItem]]

adapter = TypeAdapter(ListSubaccountsResponse)

class List(Endpoint):
  async def __call__(self, *, validate: bool = True) -> ListSubaccountsResponse:
    """Return the user's subaccounts.
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v1/account/subaccount)"""
    r = await self.authed_request('GET', '/v1/account/subaccount')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  walletId: str
  userId: str
  currency: str
  lockPeriodId: str | None
  rewardCurrency: str
  createdAt: datetime
  updatedAt: datetime

adapter = TypeAdapter(list[Entry])

class ListRewardsConfig(Endpoint):
  async def list_rewards_config(self, *, validate: bool = True) -> list[Entry]:
    """Retrieves rewards configuration for user wallets
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v1/earn/wallets/rewards/config)"""
    r = await self.authed_request('GET', '/v1/earn/wallets/rewards/config')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Literal
from bit2me.types import MillisTimestamp
from bit2me.core import Endpoint, timestamp as ts
from pydantic import TypeAdapter

adapter = TypeAdapter(list[list[float]])

class Candles(Endpoint):
  async def __call__(
    self,
    *,
    symbol: str,
    interval: Literal[1, 5, 15, 30, 60, 240, 1440],
    start_time: MillisTimestamp | datetime,
    end_time: MillisTimestamp | datetime,
    limit: float,
    validate: bool = True
  ) -> list[list[float]]:
    """Get OHLCV (open, highest, lowest, close, volume) information. The last entry in the OHLCV array is for the current.
    
    - `symbol`
    - `interval`
    - `start_time`
    - `end_time`
    - `limit`: The limit of `interval` entries (with an `interval` of 15 minutes and `limit` 4 the response is the last hour with 4 entries)
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/marketdata/GET/v1/trading/candle)"""
    params: dict = {
      'symbol': symbol,
      'interval': interval,
      'startTime': ts.dump(start_time),
      'endTime': ts.dump(end_time),
      'limit': limit,
    }
    r = await self.request('GET', '/v1/trading/candle', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import PayOrderData, PayOrderType, Phone
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class SocialPayOrderRequest(TypedDict):
  pocketId: str
  type: PayOrderType
  email: NotRequired[str]
  phone: NotRequired[Phone]
  alias: NotRequired[str]
  amount: str
  currency: str
  """Valid currency symbol"""
  note: NotRequired[str]

adapter = TypeAdapter(PayOrderData)

class Create(Endpoint):
  async def create(
    self,
    social_pay_order_request: SocialPayOrderRequest,
    *,
    validate: bool = True
  ) -> PayOrderData:
    """Creates a new payment order
    
    - `social_pay_order_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/transfers/POST/v1/social-pay/order)"""
    r = await self.authed_request(
      'POST', '/v1/social-pay/order',
      json=social_pay_order_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(BooleanResultResponse)

class Submit(Endpoint):
  async def submit(self, *, validate: bool = True) -> BooleanResultResponse:
    """Send the data to be verified (change the status to pending)
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/POST/v1/account/verify/identity)"""
    r = await self.authed_request('POST', '/v1/account/verify/identity')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[list[float | None]])

class Chart(Endpoint):
  async def chart(
    self,
    *,
    ticker: str,
    temporality: list[str] | None = None,
    validate: bool = True
  ) -> list[list[float | None]]:
    """Returns historic price chart of given ticker
    
    - `ticker`
    - `temporality`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v3/currency/chart)"""
    params: dict = {
      'ticker': ticker,
    }
    if temporality is not None:
      params['temporality'] = temporality
    r = await self.authed_request('GET', '/v3/currency/chart', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core.mixin import Router
from .movements import Movements
from .summary import Summary
from .wallets import Wallets

class Earn(Router):
  movements: Movements
  summary: Summary
  wallets: Wallets



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class KycVerificationInitV3request(TypedDict):
  locale: str
  workflowId: NotRequired[float]
  """Workflow id: 10011"""
  successUrl: NotRequired[str]
  errorUrl: NotRequired[str]

class KycVerificationInitV3response(TypedDict):
  redirectUrl: NotRequired[str]
  token: NotRequired[str]
  reference: NotRequired[str]
  provider: NotRequired[Literal['jumio', 'incode']]

adapter = TypeAdapter(KycVerificationInitV3response)

class IdentityVerification(Endpoint):
  async def identity_verification(
    self,
    kyc_verification_init_v3request: KycVerificationInitV3request,
    *,
    validate: bool = True
  ) -> KycVerificationInitV3response:
    """Init KYC verification iframe.
    Open the redirectUrl in a native browser.
    
    Do NOT open the url in webviews to avoid incompatibilities with screen or camera recording permissions.
    
    - `kyc_verification_init_v3request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/kyc/POST/v3/account/verify/identity/init)"""
    r = await self.authed_request(
      'POST', '/v3/account/verify/identity/init',
      json=kyc_verification_init_v3request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  id: NotRequired[str]
  createdAt: NotRequired[datetime]
  """When the address was created (ISO 8601)"""
  address: NotRequired[str]
  network: NotRequired[str]
  """The destination address network"""
  tag: NotRequired[str | None]

adapter = TypeAdapter(list[Entry])

class Pockets(Endpoint):
  async def pockets(
    self,
    pocket_id: str,
    network: str,
    *,
    validate: bool = True
  ) -> list[Entry]:
    """Find all address by pocket ID and Network or create one new
    
    - `pocket_id`
    - `network`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v2/wallet/pocket/{pocketId}/{network}/address)"""
    r = await self.authed_request(
      'GET', f'/v2/wallet/pocket/{pocket_id}/{network}/address'
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  pocketId: str
  orderId: str

adapter = TypeAdapter(BooleanResultResponse)

class Claim(Endpoint):
  async def claim(
    self,
    list_entry: list[Entry],
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Claim user pay orders
    
    - `list_entry`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/transfers/POST/v1/social-pay/order/claim)"""
    r = await self.authed_request(
      'POST', '/v1/social-pay/order/claim',
      json=list_entry
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Extra(TypedDict):
  symbol: str
  name: str
  slug: str
  mainwebUri: NotRequired[str]
  uriScheme: str
  hexColor: str
  assetType: str
  exponent: NotRequired[int]
  unitPriceScale: NotRequired[int]
  transactionUnitPriceScale: NotRequired[int]
  addressRegex: NotRequired[str]
  hasImage: bool
  hasPaymentRequest: bool
  isERC20Token: bool
  enabled: bool
  lite: NotRequired[bool]
  exchange: NotRequired[str]
  pairsWith: list[str]

adapter = TypeAdapter(dict[str, Extra])

class List(Endpoint):
  async def list(
    self,
    *,
    include_testnet: bool | None = None,
    show_exchange: bool | None = None,
    validate: bool = True
  ) -> dict[str, Extra]:
    """Return all assets.
    
    - `include_testnet`
    - `show_exchange`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v2/currency/assets)"""
    params = {}
    if include_testnet is not None:
      params['includeTestnet'] = include_testnet
    if show_exchange is not None:
      params['showExchange'] = show_exchange
    r = await self.authed_request('GET', '/v2/currency/assets', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  symbol: str
  """Valid currency symbol"""
  actions: list[str]
  limit: NotRequired[dict[str, Any]]

adapter = TypeAdapter(list[Entry])

class Settings(Endpoint):
  async def __call__(self, *, validate: bool = True) -> list[Entry]:
    """Get asset settings
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v1/wallet/settings/assets)"""
    r = await self.request('GET', '/v1/wallet/settings/assets')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Literal, TypedDict
from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class UpdatePersonIdentityRequest(TypedDict):
  type: Literal['none', 'idcard', 'passport', 'drivinglicense', 'visa']
  number: str
  expiryDate: datetime
  countryCode: str

adapter = TypeAdapter(BooleanResultResponse)

class PersonIdentity(Endpoint):
  async def __call__(
    self,
    update_person_identity_request: UpdatePersonIdentityRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Update user person identity
    
    - `update_person_identity_request`: The details to update user person identity
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/PUT/v1/account/users/person/identity)"""
    r = await self.authed_request(
      'put', '/v1/account/users/person/identity',
      json=update_person_identity_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import CreatedResourceIdResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class ExecuteWalletTransactionRequest(TypedDict):
  proforma: str
  concept: NotRequired[str]
  note: NotRequired[str]

adapter = TypeAdapter(CreatedResourceIdResponse)

class Execute(Endpoint):
  async def execute(
    self,
    execute_wallet_transaction_request: ExecuteWalletTransactionRequest,
    *,
    validate: bool = True
  ) -> CreatedResourceIdResponse:
    """Executes a previuosly created transaction
    
    - `execute_wallet_transaction_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/POST/v1/wallet/transaction)"""
    r = await self.authed_request(
      'POST', '/v1/wallet/transaction',
      json=execute_wallet_transaction_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any, Literal, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  fiat: dict[str, Any]
  crypto: dict[str, Any]

adapter = TypeAdapter(list[Entry])

class Rates(Endpoint):
  async def rates(
    self,
    *,
    type: Literal['all', 'fiat', 'crypto'] | None = None,
    time: str,
    validate: bool = True
  ) -> list[Entry]:
    """Return all supported exchange rates in USD.
    
    - `type`
    - `time`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v1/currency/rate)"""
    params: dict = {
      'time': time,
    }
    if type is not None:
      params['type'] = type
    r = await self.authed_request('GET', '/v1/currency/rate', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import TokenResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Session(TypedDict):
  id: str
  expirationTime: int

class SignInEmbedRequest(TypedDict):
  accessToken: str

class SignInEmbedResponse(TypedDict):
  accessToken: TokenResponse
  refreshToken: TokenResponse
  session: NotRequired[Session]

adapter = TypeAdapter(SignInEmbedResponse)

class Embed(Endpoint):
  async def embed(
    self,
    sign_in_embed_request: SignInEmbedRequest,
    *,
    validate: bool = True
  ) -> SignInEmbedResponse:
    """Generate access and refresh tokens for the embed session and subsequent API calls.
    
    - `sign_in_embed_request`: Sign-in payload
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/access/POST/v1/signin/embed)"""
    r = await self.request('POST', '/v1/signin/embed', json=sign_in_embed_request)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Extra(TypedDict):
  daily: float
  weekly: float
  monthly: float

adapter = TypeAdapter(dict[str, Extra])

class Apy(Endpoint):
  async def apy(self, *, validate: bool = True) -> dict[str, Extra]:
    """Get current annual percentage yields by currency. Value of currency is an object wich reward type as a key and annual percentage yield as value
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/GET/v2/earn/apy)"""
    r = await self.request('GET', '/v2/earn/apy')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypeVar, Generic, Any, TypedDict as _TypedDict, Annotated
from pydantic import with_config, ConfigDict, BeforeValidator
from datetime import datetime

from .exc import ValidationError
from .util import timestamp as ts

@with_config(ConfigDict(extra='allow'))
class TypedDict(_TypedDict):
  ...

Timestamp = Annotated[datetime, BeforeValidator(ts.parse)]

T = TypeVar('T')

class validator(Generic[T]):

  def __init__(self, Type: type[T]):
    from pydantic import TypeAdapter
    self.adapter = TypeAdapter(Type)
    
  def json(self, data: str | bytes | bytearray) -> T:
    from pydantic import ValidationError as PydanticValidationError
    try:
      return self.adapter.validate_json(data)
    except PydanticValidationError as e:
      raise ValidationError(*e.args) from e

  def python(self, data: Any) -> T:
    from pydantic import ValidationError as PydanticValidationError
    try:
      return self.adapter.validate_python(data)
    except PydanticValidationError as e:
      raise ValidationError(*e.args) from e
    
  def __call__(self, data) -> T:
    if isinstance(data, str | bytes | bytearray):
      return self.json(data)
    else:
      return self.python(data)



from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict
from bit2me.types import AmountCurrencyObject, Rate, TransactionSubsFeeTypeParam
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Flip(TypedDict):
  percentage: str
  amount: str
  currency: str
  """Valid currency symbol"""

class Teller(TypedDict):
  id: NotRequired[str]
  """Teller orderId"""
  feeCurrency: NotRequired[str]
  """Currency of the fees"""
  fixedFee: NotRequired[str]
  """Fixed fee"""
  variableFee: NotRequired[str]
  """Variable fee"""
  variableFeePercentage: NotRequired[str]
  """Variable fee percentage"""

class Transaction(TypedDict):
  """Network transaction"""
  hash: str
  """Network transaction hash"""
  confirmedAt: NotRequired[datetime]
  """When the transaction was confirmed (ISO 8601)."""
  confirmationCount: NotRequired[float]
  """The number of confirmations the transaction has."""

DestinationKeywords = TypedDict('DestinationKeywords', {'class': Literal['pocket', 'network', 'b2m', 'bankAccount']})
"""
  - `class`:   Type of the destination:
     - pocket: destination is another pocket
     - network: destination is an cryptocurrency address
     - b2m: destination is another Bit2Me user
     - bankAccount: destination is a bank account
    
"""

class Destination(DestinationKeywords):
  """Transaction destination"""
  currency: str
  """The currency of the destination"""
  amount: str
  """The total amount of "money" that the destination receives"""
  pocketName: NotRequired[str]
  """The destination pocket name"""
  pocketId: NotRequired[str]
  """The destination pocket ID"""
  address: NotRequired[str | None]
  """The destination address"""
  addressNetwork: NotRequired[str | None]
  """The destination address network"""
  addressInBlacklist: NotRequired[bool | None]
  """Indicates if destination address is in blacklist"""
  addressTag: NotRequired[str | None]
  rate: NotRequired[Rate]

class Fee(TypedDict):
  """Different fees for the transaction:
  network: only present when type is "withdrawal"
  flip: only present when the transaction has a currency convertion (instant)"""
  network: NotRequired[AmountCurrencyObject]
  flip: NotRequired[Flip]
  teller: NotRequired[Teller]

OriginKeywords = TypedDict('OriginKeywords', {'class': Literal['pocket', 'network', 'b2m', 'card', 'bank transfer', 'trading']})
"""
  - `class`:   Type of the origin:
     - pocket: origin is another pocket
     - network: origin is a cryptocurrency address
     - b2m: origin is another Bit2Me user
     - card: origin is a credit or debit card
     - bank transfer: origin is a bank transfer
     - trading: origin is trading balance
    
"""

class Origin(OriginKeywords):
  """Transaction origin"""
  currency: str
  """The origin currency"""
  amount: str
  """The total amount of "money" that is taken from origin."""
  pocketName: NotRequired[str | None]
  """The origin pocket name"""
  pocketId: NotRequired[str | None]
  """The origin pocket ID"""
  rate: NotRequired[Rate]

class WalletTransactionDetailResponse(TypedDict):
  date: datetime
  """When the transaction was created (ISO 8601)"""
  type: Literal['deposit', 'withdrawal', 'transfer']
  """Type of the transaction"""
  concept: NotRequired[str | None]
  """The concept of the transaction"""
  note: NotRequired[str | None]
  """Personal note of the user"""
  origin: Origin
  destination: Destination
  transaction: NotRequired[Transaction]
  fee: NotRequired[Fee]
  flip: NotRequired[Rate]
  substractFeeType: NotRequired[TransactionSubsFeeTypeParam | None]

adapter = TypeAdapter(WalletTransactionDetailResponse)

class Get(Endpoint):
  async def get(
    self,
    id: str,
    *,
    user_currency: str | None = None,
    validate: bool = True
  ) -> WalletTransactionDetailResponse:
    """Get the details of a transaction
    
    - `id`: The transaction id (returned by GET /v1/transaction)
    - `user_currency`: The user's currency (used to show a rate from it to Euro)
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v1/wallet/transaction/{id})"""
    params = {}
    if user_currency is not None:
      params['userCurrency'] = user_currency
    r = await self.authed_request(
      'GET', f'/v1/wallet/transaction/{id}',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict, deprecated
from bit2me.types import Rate
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Transaction(TypedDict):
  """Network transaction"""
  confirmedAt: NotRequired[datetime]
  """When the transaction was confirmed (ISO 8601)."""
  confirmationCount: NotRequired[float]
  """The number of confirmations the transaction has."""

DestinationKeywords = TypedDict('DestinationKeywords', {'class': Literal['pocket', 'network', 'b2m', 'bankAccount']})
"""
  - `class`:   Type of the destination:
    - pocket: destination is another pocket
    - network: destination is an cryptocurrency address
    - b2m: destination is another Bit2Me user
    - bankAccount: destination is a bank account
    
"""

class Destination(DestinationKeywords):
  """Destination of the transaction"""
  currency: NotRequired[str]
  """The destination currency"""
  amount: NotRequired[str]
  """The total amount of "money" that the destination receives"""
  pocketName: str
  """The destination pocket name"""
  pocketId: str
  """The destination pocket ID"""
  address: NotRequired[str]
  """The destination address"""
  addressNetwork: NotRequired[str]
  """The destination address network"""
  addressInBlacklist: NotRequired[bool]
  """Indicates if destination address is in blacklist"""
  rate: NotRequired[Rate]

OriginKeywords = TypedDict('OriginKeywords', {'class': Literal['pocket', 'network', 'b2m', 'card', 'bank transfer']})
"""
  - `class`:   Type of the origin:
    - pocket: origin is another pocket
    - network: origin is a cryptocurrency address
    - b2m: origin is another Bit2Me user
    - card: origin is a credit or debit card
    - bank transfer: origin is a bank transfer
    
"""

class Origin(OriginKeywords):
  """Origin of the transaction"""
  currency: NotRequired[str]
  """The currency of the origin (only present for "outcome transactions")"""
  amount: NotRequired[str]
  """The total amount of "money" that is taken from origin (only present for "outcome transactions")"""
  pocketId: str
  """The destination pocket ID of the transaction (null when class is "pocket" or "b2m")"""
  pocketName: str
  """The destination pocket name (null when class is "pocket" or "b2m")"""
  rate: NotRequired[Rate]

class DataItem(TypedDict):
  id: str
  """The transaction identifier"""
  date: str
  """When the transaction was created (ISO 8601)"""
  type: Literal['deposit', 'withdrawal', 'transfer']
  """Type of the transaction"""
  concept: str
  """The concept of the transaction"""
  origin: Origin
  destination: Destination
  transaction: NotRequired[Transaction]

class ListWalletTransactionsResponse(TypedDict):
  total: float
  """Number of returned entries"""
  data: list[DataItem]
  """Transactions returned"""

adapter = TypeAdapter(ListWalletTransactionsResponse)

class List(Endpoint):
  @deprecated('Deprecated method')
  async def list(
    self,
    *,
    pocket_id: str | None = None,
    offset: int | None = None,
    limit: int | None = None,
    keyword: str | None = None,
    type: Literal['transfer', 'withdrawal', 'deposit'] | None = None,
    subtype: Literal['purchase', 'sell', 'manual-send', 'automatic-send', 'manual-transfer', 'funding', 'receive', 'transfer', 'swap', 'reimbursement', 'earn', 'launchpad-purchase'] | None = None,
    method: Literal['pocket', 'blockchain', 'card', 'bank', 'bank-transfer', 'profit-share', 'promo-referral'] | None = None,
    from_: datetime | None = None,
    to: datetime | None = None,
    user_currency: str | None = None,
    validate: bool = True
  ) -> ListWalletTransactionsResponse:
    """Return all transactions related to a pocket. If `pocketId` is not specified, all user transactions are returned.
    
    Transactions can be paginated with `offset` and `limit`. For example, to fetch the third page with 20 records per page:
    
    ```text
    /v1/wallet/transaction?offset=40&limit=20
    ```
    
    Multiple filters can be applied when retrieving transactions. For example:
    
    ```text
    /v1/wallet/transaction?pocketId=9a2dfa96-2fb8-4b76-a9bd-83949c417540&type=deposit
    ```
    
    - `pocket_id`: The pocket identifier to get the transactions or comma delimited list of pocket IDs
    - `offset`: Specify the number of entries to be skipped (0 by default)
    - `limit`: Specify the maximum number of entries to be returned (20 by default)
    - `keyword`: Specify a list (comma-separated values) of keywords to be used to filter the query
    - `type`: Specify the transaction type to filter the entries to be returned (or comma delimited list of types)
    - `subtype`: Specify the transaction subtype to filter the entries to be returned (or comma delimited list of subtypes)
    - `method`: Specify the transaction method to filter the entries to be returned (or comma delimited list of methods)
    - `from_`: Specify the start date (ISO 8601) to filter the date (mandatory if the to parameter is set)
    - `to`: Specify the end date (ISO 8601) to filter the date (mandatory if the from parameter is set)
    - `user_currency`: The user's currency (used to show a rate from it to Euro)
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v1/wallet/transaction)"""
    params = {}
    if pocket_id is not None:
      params['pocketId'] = pocket_id
    if offset is not None:
      params['offset'] = offset
    if limit is not None:
      params['limit'] = limit
    if keyword is not None:
      params['keyword'] = keyword
    if type is not None:
      params['type'] = type
    if subtype is not None:
      params['subtype'] = subtype
    if method is not None:
      params['method'] = method
    if from_ is not None:
      params['from'] = from_
    if to is not None:
      params['to'] = to
    if user_currency is not None:
      params['userCurrency'] = user_currency
    r = await self.authed_request('GET', '/v1/wallet/transaction', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class TravelRuleInformationRequest(TypedDict):
  walletType: Literal['exchange', 'unhosted']
  """Wallet type where the user is sending the information. It coud be an exchange or selfhosted wallet."""
  exchangeName: NotRequired[str]
  """Exchange name if the operation is send to it."""
  walletOwnership: Literal['own', 'other']
  """If sending to a own or others address."""
  personType: Literal['person', 'company']
  """If sending to a person or company."""
  name: str
  """Beneficiary person name or company name."""
  surname: NotRequired[str]
  """Beneficiary person surname."""

adapter = TypeAdapter(BooleanResultResponse)

class TravelRuleOrders(Endpoint):
  async def travel_rule_orders(
    self,
    order_id: str,
    travel_rule_information_request: TravelRuleInformationRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """The Travel Rule is a regulation that requires transfers of funds, including crypto assets, to include information about the originator and the beneficiary to prevent money laundering and terrorist financing. [More information](https://support.bit2me.com/en/support/solutions/articles/35000278021-travel-rule-frequently-asked-questions)
    
    When you create a blockchain withdrawal, its status will be set to `waiting-user-information`. To proceed with the transaction, you must send the required Travel Rule parameters. If this information is not provided, the operation will remain in this status and will be automatically canceled within a few hours.
    
    - `order_id`: The order ID
    - `travel_rule_information_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/POST/v1/blockchain-manager/travel-rule-order/{orderId})"""
    r = await self.authed_request(
      'POST', f'/v1/blockchain-manager/travel-rule-order/{order_id}',
      json=travel_rule_information_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CreateEarnMovementRequest(TypedDict):
  currency: str
  """Valid currency symbol"""
  amount: str
  type: Literal['deposit', 'withdrawal']
  lockPeriod: NotRequired[str]
  withholdingAmount: NotRequired[str]
  termsAndConditions: NotRequired[str]

class CreateEarnMovementResponse(TypedDict):
  walletMovementId: str
  movementId: str

adapter = TypeAdapter(CreateEarnMovementResponse)

class Create(Endpoint):
  async def create(
    self,
    create_earn_movement_request: CreateEarnMovementRequest,
    *,
    validate: bool = True
  ) -> CreateEarnMovementResponse:
    """Create a deposit or withdrawal movement in Earn.
    
    - `create_earn_movement_request`: The details to create the movement
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/earn/POST/v1/earn/movements)"""
    r = await self.authed_request(
      'POST', '/v1/earn/movements',
      json=create_earn_movement_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import Any, NotRequired, TypedDict
from bit2me.types import (
  OffsetParam,
  OrderSide,
  OrderType,
  SortDirectionParam,
  TradeResponse
)
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class ListTradingTradesResponse(TypedDict):
  count: NotRequired[float]
  data: NotRequired[list[TradeResponse]]

adapter = TypeAdapter(ListTradingTradesResponse)

class List(Endpoint):
  async def list(
    self,
    *,
    ids: Any | None = None,
    symbol: str | None = None,
    side: OrderSide | None = None,
    order_type: OrderType | None = None,
    limit: float | None = None,
    offset: OffsetParam | None = None,
    start_time: datetime | None = None,
    end_time: datetime | None = None,
    sort: str | None = None,
    direction: SortDirectionParam | None = None,
    validate: bool = True
  ) -> ListTradingTradesResponse:
    """Get all user trades paged with a maximum page size of 50.
    The result can be filtered by dates and side optionally.
    
    - `ids`: Comma separated trade identifiers
    - `symbol`
    - `side`
    - `order_type`
    - `limit`: The maximum number of trades to fetch
    - `offset`
    - `start_time`
    - `end_time`
    - `sort`: The field to sort
    - `direction`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/trading/GET/v1/trading/trade)"""
    params = {}
    if ids is not None:
      params['ids'] = ids
    if symbol is not None:
      params['symbol'] = symbol
    if side is not None:
      params['side'] = side
    if order_type is not None:
      params['orderType'] = order_type
    if limit is not None:
      params['limit'] = limit
    if offset is not None:
      params['offset'] = offset
    if start_time is not None:
      params['startTime'] = start_time
    if end_time is not None:
      params['endTime'] = end_time
    if sort is not None:
      params['sort'] = sort
    if direction is not None:
      params['direction'] = direction
    r = await self.authed_request('GET', '/v1/trading/trade', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class FeeAmountLiquidated(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]
  converted: NotRequired[str]
  convertedCurrency: NotRequired[str]

class GuaranteeAmount(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]
  converted: NotRequired[str]
  convertedCurrency: NotRequired[str]

class GuaranteeAmountLiquidated(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]
  converted: NotRequired[str]
  convertedCurrency: NotRequired[str]

class LoanAmount(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]
  converted: NotRequired[str]
  convertedCurrency: NotRequired[str]

class PaybackAmount(TypedDict):
  value: NotRequired[str]
  currency: NotRequired[str]
  converted: NotRequired[str]
  convertedCurrency: NotRequired[str]

class Payload(TypedDict):
  ltv: NotRequired[str]
  previousLtv: NotRequired[str]
  liquidationLtv: NotRequired[str]
  guaranteeAmount: NotRequired[GuaranteeAmount]
  isCompleted: NotRequired[bool]
  loanAmount: NotRequired[LoanAmount]
  paybackAmount: NotRequired[PaybackAmount]
  guaranteeAmountLiquidated: NotRequired[GuaranteeAmountLiquidated]
  feeAmountLiquidated: NotRequired[FeeAmountLiquidated]
  isPromotional: NotRequired[bool]

class DataItem(TypedDict):
  movementId: NotRequired[str]
  orderId: NotRequired[str]
  userId: NotRequired[str]
  type: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]
  payload: NotRequired[Payload]

class ListLoanMovementsResponse(TypedDict):
  total: NotRequired[int]
  data: NotRequired[list[DataItem]]

adapter = TypeAdapter(ListLoanMovementsResponse)

class Movements(Endpoint):
  async def __call__(
    self,
    *,
    order_id: str | None = None,
    limit: float | None = None,
    offset: float | None = None,
    validate: bool = True
  ) -> ListLoanMovementsResponse:
    """Get loan user movements
    
    - `order_id`
    - `limit`
    - `offset`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/GET/v1/loan/movements)"""
    params = {}
    if order_id is not None:
      params['orderId'] = order_id
    if limit is not None:
      params['limit'] = limit
    if offset is not None:
      params['offset'] = offset
    r = await self.authed_request('GET', '/v1/loan/movements', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core.mixin import Router
from .create import Create
from .list import List
from .two_factor import TwoFactor

class Subaccounts(Router):
  create: Create
  list: List
  two_factor: TwoFactor



from datetime import datetime
from typing_extensions import Literal, NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Benefit(TypedDict):
  tier: NotRequired[int]
  levelId: NotRequired[str]
  quantity: NotRequired[str]
  percentage: NotRequired[int]
  currency: NotRequired[str]
  amount: NotRequired[float]

class Converted(TypedDict):
  amount: NotRequired[str]
  amountAfterFees: NotRequired[str]
  currency: NotRequired[str]

class Converted2(TypedDict):
  amount: NotRequired[str]
  amountAfterFees: NotRequired[str]
  currency: NotRequired[str]

class Fixed(TypedDict):
  amount: NotRequired[str]
  currency: NotRequired[str]

class Flip(TypedDict):
  percentage: NotRequired[str]
  amount: NotRequired[str]
  currency: NotRequired[str]

class Network(TypedDict):
  amount: NotRequired[str]
  currency: NotRequired[str]

class Pair(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Pair2(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Pair3(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Pair4(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Pair5(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Pair6(TypedDict):
  base: NotRequired[str]
  quote: NotRequired[str]

class Phone2(TypedDict):
  number: NotRequired[str]
  countryCode: NotRequired[str]

class Phone3(TypedDict):
  number: NotRequired[str]
  countryCode: NotRequired[str]

class Transaction(TypedDict):
  hash: NotRequired[str]
  confirmedAt: NotRequired[str]
  confirmationCount: NotRequired[int]

class UserAmount(TypedDict):
  currency: NotRequired[str]
  value: NotRequired[str]

class UserAmount2(TypedDict):
  currency: NotRequired[str]
  value: NotRequired[str]

class UserAmount3(TypedDict):
  currency: NotRequired[str]
  amount: NotRequired[str]

class Variable(TypedDict):
  percentage: NotRequired[str]
  amount: NotRequired[str]
  currency: NotRequired[str]

class Rate10(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair6]

class Rate3(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair]

class Rate5(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair2]

class Rate7(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair3]

class Rate8(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair4]

class Rate9(TypedDict):
  value: NotRequired[str]
  pair: NotRequired[Pair5]

class Teller(TypedDict):
  fixed: NotRequired[Fixed]
  variable: NotRequired[Variable]
  id: NotRequired[str]
  feeCurrency: NotRequired[str]
  fixedFee: NotRequired[str]
  variableFee: NotRequired[str]
  variableFeePercentage: NotRequired[str]

class Destination2(TypedDict):
  currency: NotRequired[str]
  amount: NotRequired[str]
  rate: NotRequired[Rate10]

class Fee(TypedDict):
  network: NotRequired[Network]
  flip: NotRequired[Flip]
  teller: NotRequired[Teller]

class Flip2(TypedDict):
  rate: NotRequired[Rate8]

class Rate2(TypedDict):
  rate: NotRequired[Rate3]

class Rate4(TypedDict):
  rate: NotRequired[Rate5]

class Rate6(TypedDict):
  rate: NotRequired[Rate7]

class UserRate(TypedDict):
  rate: NotRequired[Rate9]

class Company(TypedDict):
  destination: NotRequired[Destination2]

class Denomination(TypedDict):
  amount: NotRequired[str]
  currency: NotRequired[str]
  rate: NotRequired[Rate2]

DestinationKeywords = TypedDict('DestinationKeywords', {'class': str})

class Destination(DestinationKeywords):
  currency: NotRequired[str]
  pocketName: NotRequired[str | None]
  pocketId: NotRequired[str | None]
  bankAccount: NotRequired[str]
  email: NotRequired[str]
  phone: NotRequired[Phone3]
  alias: NotRequired[str]
  fullName: NotRequired[str]
  address: NotRequired[str | None]
  addressNetwork: NotRequired[str | None]
  addressTag: NotRequired[str | None]
  addressInBlacklist: NotRequired[bool | None]
  amount: NotRequired[str]
  amountAfterFees: NotRequired[str]
  rate: NotRequired[Rate6]
  converted: NotRequired[Converted2]
  userAmount: NotRequired[UserAmount2]
  userId: NotRequired[str]

OriginKeywords = TypedDict('OriginKeywords', {'class': str})

class Origin(OriginKeywords):
  currency: NotRequired[str]
  pocketName: NotRequired[str | None]
  pocketId: NotRequired[str | None]
  bankAccount: NotRequired[str]
  email: NotRequired[str]
  phone: NotRequired[Phone2]
  alias: NotRequired[str]
  fullName: NotRequired[str]
  address: NotRequired[str | None]
  addressNetwork: NotRequired[str | None]
  addressTag: NotRequired[str | None]
  addressInBlacklist: NotRequired[bool | None]
  amount: NotRequired[str]
  amountAfterFees: NotRequired[str]
  rate: NotRequired[Rate4]
  converted: NotRequired[Converted]
  userAmount: NotRequired[UserAmount]
  userId: NotRequired[str]

class DataItem(TypedDict):
  id: NotRequired[str]
  note: NotRequired[str]
  date: NotRequired[datetime]
  completedAt: NotRequired[datetime]
  canceledAt: NotRequired[datetime | None]
  concept: NotRequired[str]
  type: NotRequired[str]
  subtype: NotRequired[str]
  method: NotRequired[str]
  status: NotRequired[str]
  substractFeeType: NotRequired[str]
  denomination: NotRequired[Denomination]
  frequency: NotRequired[str]
  isInitialRecurringOrder: NotRequired[bool]
  origin: NotRequired[Origin]
  destination: NotRequired[Destination]
  transaction: NotRequired[Transaction]
  fee: NotRequired[Fee]
  flip: NotRequired[Flip2]
  benefit: NotRequired[Benefit]
  userRate: NotRequired[UserRate]
  userAmount: NotRequired[UserAmount3]
  instantId: NotRequired[str]
  company: NotRequired[Company]

class ListWalletTransactionsV2response(TypedDict):
  data: NotRequired[list[DataItem]]
  total: NotRequired[int]

adapter = TypeAdapter(ListWalletTransactionsV2response)

class Transactions(Endpoint):
  async def transactions(
    self,
    *,
    offset: int | None = None,
    limit: int | None = None,
    year: int | None = None,
    currency: str | None = None,
    operation: Literal['purchase', 'sell', 'swap', 'deposit', 'withdrawal', 'deposit-earn', 'withdrawal-earn', 'deposit-trading', 'withdrawal-trading', 'send-pay', 'receive-pay', 'purchase-bcard', 'reimburse-bcard', 'send', 'receive'] | None = None,
    validate: bool = True
  ) -> ListWalletTransactionsV2response:
    """Get user transactions.
    Transactions can be paginated using **offset** and **limit** parameters. For example, to get the third page and show 20 registers per page you would use the following query:
    
    ``` /v2/wallet/transaction?offset=40&limit=20 ```
    
    - `offset`: Specify the number of entries to be skipped (0 by default)
    - `limit`: Specify the maximum number of entries to be returned (20 by default)
    - `year`: Filter movements to this specific year
    - `currency`: Currency in which movements has been involved
    - `operation`: Operation to filter
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v2/wallet/transaction)"""
    params = {}
    if offset is not None:
      params['offset'] = offset
    if limit is not None:
      params['limit'] = limit
    if year is not None:
      params['year'] = year
    if currency is not None:
      params['currency'] = currency
    if operation is not None:
      params['operation'] = operation
    r = await self.authed_request('GET', '/v2/wallet/transaction', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any
from bit2me.types import WalletResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[WalletResponse])

class Balance(Endpoint):
  async def __call__(
    self,
    *,
    symbols: Any | None = None,
    validate: bool = True
  ) -> list[WalletResponse]:
    """Retrieve balances of all wallets and blocked balances in active orders.
    
    - `symbols`: Comma separated symbols of the wallets that you want to retrieve
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/funding/GET/v1/trading/wallet/balance)"""
    params = {}
    if symbols is not None:
      params['symbols'] = symbols
    r = await self.authed_request(
      'GET', '/v1/trading/wallet/balance',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.core.mixin import Router
from .account import Account
from .currency import Currency
from .earn import Earn
from .loan import Loan
from .trading import Trading
from .wallet import Wallet

class V2(Router):
  account: Account
  currency: Currency
  earn: Earn
  loan: Loan
  trading: Trading
  wallet: Wallet



from .util import timestamp, round2tick, trunc2tick
from .exc import Error, NetworkError, RateLimited, ValidationError, ApiError
from .validation import validator, TypedDict, Timestamp
from .http import HttpClient, HttpMixin, AuthHttpClient, AuthHttpMixin
from .mixin import Endpoint, Router, BIT2ME_API_URL

__all__ = [
  'timestamp', 'round2tick', 'trunc2tick',
  'Error', 'NetworkError', 'RateLimited', 'ValidationError', 'ApiError',
  'validator', 'TypedDict', 'Timestamp',
  'HttpClient', 'HttpMixin', 'AuthHttpClient', 'AuthHttpMixin',
  'Endpoint', 'Router',
  'BIT2ME_API_URL',
]



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class DataItem(TypedDict):
  id: str
  description: str
  professionCode: str
  langCode: str

class ListProfessionsResponse(TypedDict):
  total: int
  data: list[DataItem]

adapter = TypeAdapter(ListProfessionsResponse)

class Professions(Endpoint):
  async def __call__(
    self,
    *,
    lang_code: str | None = None,
    validate: bool = True
  ) -> ListProfessionsResponse:
    """Return all professions. Values are lowercase.
    
    - `lang_code`: Lang code
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v1/account/professions)"""
    params = {}
    if lang_code is not None:
      params['langCode'] = lang_code
    r = await self.authed_request('GET', '/v1/account/professions', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import BooleanResultResponse, PocketColor
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class UpdateWalletPocketRequest(TypedDict):
  id: str
  name: NotRequired[str]
  color: NotRequired[PocketColor]

adapter = TypeAdapter(BooleanResultResponse)

class Update(Endpoint):
  async def update(
    self,
    update_wallet_pocket_request: UpdateWalletPocketRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Update pocket data
    
    - `update_wallet_pocket_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/PUT/v1/wallet/pocket)"""
    r = await self.authed_request(
      'put', '/v1/wallet/pocket',
      json=update_wallet_pocket_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class GuaranteeCurrenciesItem(TypedDict):
  currencyConfigurationGuaranteeId: NotRequired[str]
  currency: NotRequired[str]
  enabled: NotRequired[bool]
  liquidationLtv: NotRequired[str]
  initialLtv: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]

class LoanCurrenciesItem(TypedDict):
  currencyConfigurationLoanId: NotRequired[str]
  currency: NotRequired[str]
  enabled: NotRequired[bool]
  liquidity: NotRequired[str]
  liquidityStatus: NotRequired[str]
  apr: NotRequired[str]
  minimumAmount: NotRequired[str]
  maximumAmount: NotRequired[str]
  createdAt: NotRequired[datetime]
  updatedAt: NotRequired[datetime]

class LoanCurrencyConfigResponse(TypedDict):
  loanCurrencies: NotRequired[list[LoanCurrenciesItem]]
  guaranteeCurrencies: NotRequired[list[GuaranteeCurrenciesItem]]

adapter = TypeAdapter(LoanCurrencyConfigResponse)

class Currencies(Endpoint):
  async def currencies(self, *, validate: bool = True) -> LoanCurrencyConfigResponse:
    """Get the configuration of loan currencies. This includes which currencies are available for loans and which can be used as guarantee
    
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/loan/GET/v2/loan/currency/configuration)"""
    r = await self.authed_request('GET', '/v2/loan/currency/configuration')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class CurrencyMarketDataV3response(TypedDict):
  marketCap: str
  totalVolume: str
  fullyDilutedMarketCap: str
  maxSupply: int
  totalSupply: int

adapter = TypeAdapter(CurrencyMarketDataV3response)

class MarketData(Endpoint):
  async def market_data(
    self,
    symbol: str,
    *,
    validate: bool = True
  ) -> CurrencyMarketDataV3response:
    """Returns the market data for given currency in USD
    
    - `symbol`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v3/currency/market-data/{symbol})"""
    r = await self.authed_request('GET', f'/v3/currency/market-data/{symbol}')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from datetime import datetime
from typing_extensions import TypedDict
from bit2me.types import PocketColor
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Entry(TypedDict):
  id: str
  name: str
  color: PocketColor
  currency: str
  """Valid currency symbol"""
  balance: str
  blockedBalance: str
  createdAt: datetime
  """When the pocket was created (ISO 8601)"""

adapter = TypeAdapter(list[Entry])

class Get(Endpoint):
  async def get(self, *, id: str | None = None, validate: bool = True) -> list[Entry]:
    """Get the data of a specific pocket or the data of all pockets of the user (if the *id* param is not set)
    
    - `id`: With this parameter you can specify the pocket you want to get the data from.
    If this parameter is not set, all pockets of the user are returned
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/wallet/GET/v1/wallet/pocket)"""
    params = {}
    if id is not None:
      params['id'] = id
    r = await self.authed_request('GET', '/v1/wallet/pocket', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import TokenResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class SignInApiKeyRequest(TypedDict):
  userId: NotRequired[str]

class SignInApiKeyResponse(TypedDict):
  accessToken: TokenResponse

adapter = TypeAdapter(SignInApiKeyResponse)

class ApiKey(Endpoint):
  async def api_key(
    self,
    sign_in_api_key_request: SignInApiKeyRequest,
    *,
    validate: bool = True
  ) -> SignInApiKeyResponse:
    """Authorize a user with an API key.
    
    - `sign_in_api_key_request`: Sign-in payload
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/access/POST/v1/signin/apikey)"""
    r = await self.authed_request(
      'POST', '/v1/signin/apikey',
      json=sign_in_api_key_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Point(TypedDict):
  time: int
  price: str
  interval: str
  marketCap: NotRequired[str]
  totalVolume: NotRequired[str]
  fullyDilutedMarketCap: NotRequired[str]
  maxSupply: NotRequired[str]
  totalSupply: NotRequired[str]

adapter = TypeAdapter(dict[str, dict[str, list[Point]]])

class Ticker(Endpoint):
  async def ticker(
    self,
    symbol: str,
    *,
    rate_currency: str | None = None,
    interval: list[str] | None = None,
    extended: bool | None = None,
    validate: bool = True
  ) -> dict[str, dict[str, list[Point]]]:
    """Return ticker data for the selected cryptocurrency.
    
    - `symbol`
    - `rate_currency`
    - `interval`
    - `extended`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/market/GET/v3/currency/ticker/{symbol})"""
    params = {}
    if rate_currency is not None:
      params['rateCurrency'] = rate_currency
    if interval is not None:
      params['interval'] = interval
    if extended is not None:
      params['extended'] = extended
    r = await self.authed_request(
      'GET', f'/v3/currency/ticker/{symbol}',
      params=params
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import Any, Literal, NotRequired, TypedDict
from bit2me.types import TransactionSubsFeeTypeParam
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class Benefit(TypedDict):
  amount: str | float
  currency: str
  percentage: float
  tier: float
  levelId: NotRequired[str]

class Destination(TypedDict):
  type: Literal['pocket']
  value: str

class Method(TypedDict):
  type: Literal['creditcard', 'google-pay', 'apple-pay', 'hub-latam']
  params: NotRequired[dict[str, Any]]

class TellerCreateOrderProformaRequest(TypedDict):
  amount: str
  description: NotRequired[str]
  currency: str
  feeType: NotRequired[TransactionSubsFeeTypeParam]
  orderType: Literal['deposit', 'purchase']
  destination: NotRequired[Destination]
  method: Method

class TellerCreateOrderProformaResponse(TypedDict):
  orderId: str
  denomination: dict[str, Any]
  currency: str
  amountBeforeFee: str
  amountAfterFee: str
  fixedFee: str
  variableFee: str
  variableFeePercentage: str
  benefit: NotRequired[Benefit]

adapter = TypeAdapter(TellerCreateOrderProformaResponse)

class Preview(Endpoint):
  async def preview(
    self,
    teller_create_order_proforma_request: TellerCreateOrderProformaRequest,
    *,
    validate: bool = True
  ) -> TellerCreateOrderProformaResponse:
    """Create a deposit proforma. To execute the resulting order, confirm it with `POST /v1/teller/order`.
    
    - `teller_create_order_proforma_request`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/funding/POST/v1/teller/order/proforma)"""
    r = await self.authed_request(
      'POST', '/v1/teller/order/proforma',
      json=teller_create_order_proforma_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import TypeVar, Any
import os
from dataclasses import dataclass
import httpx

from .http import HttpMixin, AuthHttpMixin, AuthHttpClient, BIT2ME_API_URL
from .exc import ApiError

T = TypeVar('T')

@dataclass
class Endpoint(AuthHttpMixin):
  default_validate: bool = True

  def should_validate(self, validate_param: bool | None = None) -> bool:
    return self.default_validate if validate_param is None else validate_param

  def raise_error(self, response: httpx.Response):
    try:
      payload: Any = response.json()
    except:
      payload = response.text
    raise ApiError.of(response.status_code, payload)

  @classmethod
  def new(
    cls, api_key: str | None = None, api_secret: str | None = None, *,
    base_url: str = BIT2ME_API_URL, validate: bool = True,
  ):
    if api_key is None:
      api_key = os.environ['BIT2ME_API_KEY']
    if api_secret is None:
      api_secret = os.environ['BIT2ME_SECRET_KEY']
    client = AuthHttpClient(api_key=api_key, api_secret=api_secret)
    return cls(base_url=base_url, http=client, default_validate=validate)

  @classmethod
  def public(cls, *, base_url: str = BIT2ME_API_URL, validate: bool = True):
    return cls.new('', '', base_url=base_url, validate=validate)

@dataclass
class Router(Endpoint):
  def __post_init__(self):
    for field, cls in self.__annotations__.items():
      if issubclass(cls, Endpoint) or issubclass(cls, Router):
        setattr(self, field, cls(base_url=self.base_url, http=self.http, default_validate=self.default_validate))



from .get import Get
from .get_rewards_config import GetRewardsConfig
from .get_rewards_summary import GetRewardsSummary
from .list_movements import ListMovements
from .list_rewards_config import ListRewardsConfig
from .update_rewards_config import UpdateRewardsConfig

class Wallets(Get, GetRewardsConfig, GetRewardsSummary, ListMovements, ListRewardsConfig, UpdateRewardsConfig):
  ...



from bit2me.types import OrderSide
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[list[OrderSide | float]])

class GetLast(Endpoint):
  async def get_last(
    self,
    *,
    symbol: str,
    limit: float | None = None,
    validate: bool = True
  ) -> list[list[OrderSide | float]]:
    """Get a list of last trades. Returns the last 50 trades by default.
    
    - `symbol`
    - `limit`: The number of trades to retrieve
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/marketdata/GET/v1/trading/trade/last)"""
    params: dict = {
      'symbol': symbol,
    }
    if limit is not None:
      params['limit'] = limit
    r = await self.request('GET', '/v1/trading/trade/last', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



import time
from datetime import datetime
from decimal import Decimal, ROUND_HALF_DOWN, ROUND_FLOOR, ROUND_CEILING

class timestamp:
  @staticmethod
  def parse(time: int | str | datetime) -> datetime:
    return time if isinstance(time, datetime) else datetime.fromtimestamp(int(time)/1e3)
  
  @staticmethod
  def dump(dt: datetime | int) -> int:
    return int(1e3*dt.timestamp()) if isinstance(dt, datetime) else dt
  
  @staticmethod
  def now() -> int:
    return int(time.time() * 1e3)
    

def round2tick(x: Decimal, tick_size: Decimal) -> Decimal:
  r = (x / tick_size).quantize(Decimal('1.'), rounding=ROUND_HALF_DOWN) * tick_size
  return r.normalize()

def trunc2tick(x: Decimal, tick_size: Decimal) -> Decimal:
  r = (x / tick_size).to_integral_value(rounding=ROUND_FLOOR) * tick_size
  return r.normalize()

def ceil2tick(x: Decimal, tick_size: Decimal) -> Decimal:
  r = (x / tick_size).to_integral_value(rounding=ROUND_CEILING) * tick_size
  return r.normalize()


from bit2me.core.mixin import Router
from .account import Account
from .blockchain_manager import BlockchainManager
from .currency import Currency
from .earn import Earn
from .loan import Loan
from .misc import Misc
from .signin import Signin
from .social_pay import SocialPay
from .teller import Teller
from .trading import Trading
from .verifier import Verifier
from .wallet import Wallet

class V1(Router):
  account: Account
  blockchain_manager: BlockchainManager
  currency: Currency
  earn: Earn
  loan: Loan
  misc: Misc
  signin: Signin
  social_pay: SocialPay
  teller: Teller
  trading: Trading
  verifier: Verifier
  wallet: Wallet



from bit2me.types import UserAddress
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[UserAddress])

class List(Endpoint):
  async def list(
    self,
    *,
    address_id: str | None = None,
    is_residence: bool | None = None,
    validate: bool = True
  ) -> list[UserAddress]:
    """Get the addresses of a specific user
    
    - `address_id`: The address id
    - `is_residence`: Value of the isResidence field
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/GET/v2/account/address)"""
    params = {}
    if address_id is not None:
      params['addressId'] = address_id
    if is_residence is not None:
      params['isResidence'] = is_residence
    r = await self.authed_request('GET', '/v2/account/address', params=params)
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from typing_extensions import NotRequired, TypedDict
from bit2me.types import BooleanResultResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

class SettingsEnableTwoFactorEndRequest(TypedDict):
  totp: str
  sessionId: NotRequired[str]
  alwaysRequired: NotRequired[bool]

adapter = TypeAdapter(BooleanResultResponse)

class EnableFinish(Endpoint):
  async def enable_finish(
    self,
    settings_enable_two_factor_end_request: SettingsEnableTwoFactorEndRequest,
    *,
    validate: bool = True
  ) -> BooleanResultResponse:
    """Finish the 2FA enablement flow by confirming that the client and server TOTP values match.
    
    - `settings_enable_two_factor_end_request`: The details to end the process
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/doc#tag/account/PUT/v1/account/settings/sfa)"""
    r = await self.authed_request(
      'put', '/v1/account/settings/sfa',
      json=settings_enable_two_factor_end_request
    )
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()



from bit2me.types import TradeResponse
from pydantic import TypeAdapter
from bit2me.core import Endpoint

adapter = TypeAdapter(list[TradeResponse])

class ListTrades(Endpoint):
  async def list_trades(self, id: str, *, validate: bool = True) -> list[TradeResponse]:
    """Get all trades associated to the order.
    
    - `id`
    - `validate`: Whether to validate the response against the expected schema (default: True) (default: None)
    
    [Official docs](https://api.bit2me.com/trading-spot-rest#tag/trading/GET/v1/trading/order/{id}/trades)"""
    r = await self.authed_request('GET', f'/v1/trading/order/{id}/trades')
    
    if r.status_code != 200:
      self.raise_error(r)
    return adapter.validate_json(r.text) if self.should_validate(validate) else r.json()
